from sympy import (Symbol, symbols, Matrix, sin, cos, asin, diff, sqrt, S,
                   diag, Eq, hessian, Function, flatten, Tuple, im, pi, latex,
                   dsolve, solve, fraction, factorial, Subs, Number, oo, Abs,
                   N, solveset, atan)

from sympy.physics.mechanics import dynamicsymbols, ReferenceFrame, Point
from sympy.physics.vector import vpprint, vlatex
from ...dynamics import LagrangesDynamicSystem, HarmonicOscillator, mech_comp

from ..elements import MaterialPoint, Spring, GravitationalForce, Disk, RigidBody2D, Damper, PID, Excitation, Force, base_frame, base_origin
from ...continuous import ContinuousSystem, PlaneStressProblem

import base64
import random
import IPython as IP
import numpy as np
import inspect

from .pendulum import Pendulum, PendulumKinematicExct
from .principles import ComposedSystem, NonlinearComposedSystem, base_frame, base_origin, REPORT_COMPONENTS_LIST
from .tmd import TunedMassDamperRelativeMotion
from ...utilities.components.mech import en as mech_comp
from .trolley import ThreeSprings, ForcedTrolleyWithThreeSprings
from .disk import RollingDisk

from pint import UnitRegistry
ureg = UnitRegistry()

from functools import cached_property, lru_cache


### NotODE
class TrolleysWithSprings(NonlinearComposedSystem):
    
    scheme_name = 'nonlin_trolley.PNG'
    real_name = 'nonlin_trolley_real.PNG'

    m_1=Symbol('m_1', positive=True)
    m_2=Symbol('m_2', positive=True)
    k=Symbol('k', positive=True)
    #Omega=Symbol('Omega', positive=True)
    #F=Symbol('F', positive=True)
    x_1=dynamicsymbols('x_1')
    x_2=dynamicsymbols('x_2')
   
    def __init__(self,
                 m_1=None,
                 m_2=None,
                 k=None,
                 #Omega=None,
                 #F=None,
                 x_1=None,
                 x_2=None,
                 ivar=Symbol('t'),
                 **kwargs):

        if m_1 is not None: self.m_1 = m_1
        if m_2 is not None: self.m_2 = m_2
        if k is not None: self.k= k
        #if Omega is not None: self.Omega = Omega
        #if F is not None: self.F = F
        if x_1 is not None: self.x_1 = x_1
        if x_2 is not None: self.x_2 = x_2

        self.qs = [self.x_1, self.x_2]
        self._init_from_components(**kwargs)


    @property
    def components(self):

        components = {}

        self._trolley1 = MaterialPoint(self.m_1, self.x_1, qs=[self.x_1])
        self._spring1 = Spring(self.k, pos1 = self.x_1, qs=[self.x_1])
        #self._force = Force(self.F*sin(self.Omega*self.ivar), pos1 = self.x_1, qs=[self.x_1])

        self._spring12 = Spring(self.k, pos1 = self.x_1, pos2 = self.x_2, qs=[self.x_1, self.x_2])
        self._trolley2 = MaterialPoint(self.m_2, self.x_2, qs=[self.x_2])
        self._spring2 = Spring(self.k, pos1 = self.x_2, qs=[self.x_2])


        components['trolley_1'] = self._trolley1
        components['spring_1'] = self._spring1
        components['spring_12'] = self._spring12
        components['trolley_2'] = self._trolley2
        components['spring_2'] = self._spring2
        #components['force'] = self._force
        
        return components



class ForcedNonLinTrolleysWithSprings(ComposedSystem): ### 3 ODE
    
    scheme_name = 'nonlin_trolley.PNG'
    real_name = 'nonlin_trolley_real.PNG'

    m_1=Symbol('m_1', positive=True)
    m_2=Symbol('m_2', positive=True)
    k=Symbol('k', positive=True)
    Omega=Symbol('Omega', positive=True)
    F=Symbol('F', positive=True)
    G=Symbol('G', positive=True)
    x_1=dynamicsymbols('x_1')
    x_2=dynamicsymbols('x_2')
    
    m0=Symbol('m_0', positive=True)
    k0=Symbol('k_0', positive=True)
    Omega0=Symbol('Omega_0', positive=True)
    F0=Symbol('F_0', positive=True)
   
    def __init__(self,
                 m_1=None,
                 m_2=None,
                 k=None,
                 Omega=None,
                 F=None,
                 m0=None,
                 k0=None,
                 Omega0=None,
                 F0=None,
                 G=None,
                 x_1=None,
                 x_2=None,
                 ivar=Symbol('t'),
                 **kwargs):

        if m_1 is not None: self.m_1 = m_1
        if m_2 is not None: self.m_2 = m_2
        if k is not None: self.k= k
        if Omega is not None: self.Omega = Omega
        if F is not None: self.F = F
        if G is not None: self.G = G
        if x_1 is not None: self.x_1 = x_1
        if x_2 is not None: self.x_2 = x_2
            
        if m0 is not None: self.m0 = m0
        if k0 is not None: self.k0= k0
        if Omega0 is not None: self.Omega0 = Omega0
        if F0 is not None: self.F0 = F0

        self.qs = [self.x_1, self.x_2]
        self._init_from_components(**kwargs)

    @property
    def components(self):

        components = {}

        self._trolley1 = MaterialPoint(self.m_1, self.x_1, qs=[self.x_1])
        self._spring1 = Spring(self.k, pos1 = self.x_1, qs=[self.x_1])
        self._force = Force(self.F*sin(self.Omega*self.ivar) + self.G, pos1 = self.x_1, qs=[self.x_1])

        self._spring12 = Spring(self.k, pos1 = self.x_1, pos2 = self.x_2, qs=[self.x_1, self.x_2])
        self._trolley2 = MaterialPoint(self.m_2, self.x_2, qs=[self.x_2])
        self._spring2 = Spring(self.k, pos1 = self.x_2, qs=[self.x_2])


        components['trolley_1'] = self._trolley1
        components['spring_1'] = self._spring1
        components['spring_12'] = self._spring12
        components['trolley_2'] = self._trolley2
        components['spring_2'] = self._spring2
        components['force'] = self._force
        
        return components

    def get_default_data(self):

        m0, k0, F0 =  self.m0, self.k0, self.F0
        
        default_data_dict = {
            self.m_1 : [S.One * m0 * no for no in range(10,20)],
            self.m_2 : [S.One * m0 * no for no in range(20,30)],
            self.k : [S.One * k0 * no for no in range(30,50)],
            self.F : [S.One * F0 * no for no in range(1,10)]
        }
        default_data_dict.update({self.G: [4*default_data_dict[self.F][0]*cos(0.5*default_data_dict[self.Omega][0]*self.ivar) , default_data_dict[self.F][0]*cos(0.75*default_data_dict[self.Omega][0]*self.ivar)**2 , 1.5*default_data_dict[self.F][0]*2*cos(1.25*default_data_dict[self.Omega][0]*self.ivar) , 3*default_data_dict[self.F][0]*cos(2*default_data_dict[self.Omega][0]*self.ivar)**2]})

        return default_data_dict
    
    def get_numerical_data(self):

        default_data_dict = {
            self.m_1 : [100],
            self.m_2 : [200],
            self.k : [50],
            #self.c : [100],
            self.Omega : [0.5 * 3.14, 1 * 3.14, 2 * 3.14, 4 * 3.14],
            self.F : [0.5 * 100, 1 * 100, 2 * 100, 4 * 100]
        }
        default_data_dict.update({self.G: [4*default_data_dict[self.F][0]*cos(0.5*default_data_dict[self.Omega][0]*self.ivar) , default_data_dict[self.F][0]*cos(0.75*default_data_dict[self.Omega][0]*self.ivar)**2 , 1.5*default_data_dict[self.F][0]*2*cos(1.25*default_data_dict[self.Omega][0]*self.ivar) , 3*default_data_dict[self.F][0]*cos(2*default_data_dict[self.Omega][0]*self.ivar)**2]})

        return default_data_dict

    def symbols_description(self):
        self.sym_desc_dict = {
            self.m_1: r'masa wózka nr 1',
            self.m_2: r'masa wózka nr 2',
            self.k: r'sztywność sprężyny',
            self.Omega: r'częstotliwość wymuszenia',
            self.F: r'wartość stałej siły wymuszającej',
            self.G: r'wartość zmiennej siły wymuszającej',
            self.x_1: 'przemieszczenie wózka nr 1',
            self.x_2: 'przemieszczenie wózka nr 2',
        }
        return self.sym_desc_dict
    
    def unit_dict(self):
        self.sym_desc_dict = {
            self.m_1: r'masa wózka nr 1',
            self.m_2: r'masa wózka nr 2',
            self.k: r'sztywność sprężyny',
            self.Omega: r'częstotliwość wymuszenia',
            self.F: r'wartość stałej siły wymuszającej',
            self.G: r'wartość zmiennej siły wymuszającej',
            self.x_1: 'przemieszczenie wózka nr 1',
            self.x_2: 'przemieszczenie wózka nr 2',
        }
        return self.sym_desc_dict

    @property
    def _report_components(self):
        
        comp_list=[
        *REPORT_COMPONENTS_LIST,
#         mech_comp.TitlePageComponent,
#         mech_comp.SchemeComponent,
#         mech_comp.ExemplaryPictureComponent,
#         mech_comp.KineticEnergyComponent,
#         mech_comp.PotentialEnergyComponent,
#         mech_comp.LagrangianComponent,
#         mech_comp.GoverningEquationComponent,
#         mech_comp.FundamentalMatrixComponent,
#         mech_comp.GeneralSolutionComponent,
        #mech_comp.SteadySolutionComponent,
        ]
        return comp_list
    
    
class ForcedDampedTrolleysWithSprings(ComposedSystem):

    scheme_name = 'MDOF_Damped_Trolleys_With_Springs.png'
    real_name = 'two_trolleys_damped_with_springs_real.png'

    m_1=Symbol('m_1', positive=True)
    m_2=Symbol('m_2', positive=True)
    m=Symbol('m', positive=True)
    R=Symbol('R', positive=True)
    k_l=Symbol('k_l', positive=True)
    k_r=Symbol('k_r', positive=True)
    k_c=Symbol('k_c', positive=True)
    c_l=Symbol('c_l', positive=True)
    c_r=Symbol('c_r', positive=True)
    c_c=Symbol('c_c', positive=True)
    Omega=Symbol('Omega', positive=True)
    F=Symbol('F', positive=True)
    x_l=dynamicsymbols('x_l')
    x_r=dynamicsymbols('x_r')

    def __init__(self,
                 R=None,
                 m_1=None,
                 m_2=None,
                 m=None,
                 k_l=None,
                 k_r=None,
                 k_c=None,
                 c_l=None,
                 c_r=None,
                 c_c=None,
                 Omega=None,
                 F=None,
                 x_l=None,
                 x_r=None,
                 ivar=Symbol('t'),
                 **kwargs):

        if m_1 is not None: self.m_1 = m_1
        if m_2 is not None: self.m_2 = m_2
        if m is not None: self.m = m
        if k_l is not None: self.k_l = k_l
        if k_r is not None: self.k_r = k_r
        if k_c is not None: self.k_c = k_c
        if c_l is not None: self.c_l = c_l
        if c_r is not None: self.c_r = c_r
        if c_c is not None: self.c_c = c_c
        if Omega is not None: self.Omega = Omega
        if F is not None: self.F = F
        if x_l is not None: self.x_l = x_l
        if x_r is not None: self.x_r = x_r
        self.ivar=ivar

        self.qs = [self.x_l, self.x_r]
        self._init_from_components(**kwargs)

    @cached_property
    def components(self):

        components = {}

        self._trolley1 = MaterialPoint(self.m_1, self.x_l, qs=[self.x_l])
        self._wheel11 = MaterialPoint(self.m, self.x_l, qs = [self.x_l])
        self._wheel11_disk = Disk(I=S.Half*self.m*self.R**2, pos1=self.x_l/self.R, qs=[self.x_l])
        self._wheel12= MaterialPoint(self.m, self.x_l, qs = [self.x_l])
        self._wheel12_disk = Disk(I=S.Half*self.m*self.R**2, pos1=self.x_l/self.R, qs=[self.x_l])
        
        self._spring1_top = Spring(self.k_l, pos1 = self.x_l, qs=[self.x_l])
        self._spring1_bottom = Spring(self.k_l, pos1 = self.x_l, qs=[self.x_l])
        
        self._damper1 = Damper(self.c_l, pos1 = self.x_l, qs=[self.x_l])
        self._force = Force(self.F*sin(self.Omega*self.ivar) + self.F, pos1 = self.x_l, qs=[self.x_l])

        self._spring12 = Spring(2*self.k_c, pos1 = self.x_l, pos2 = self.x_r, qs=[self.x_l, self.x_r])
        self._damper12 = Damper(self.c_c, pos1 = self.x_l, pos2 = self.x_r, qs=[self.x_l, self.x_r])
        self._wheel21 = MaterialPoint(self.m, self.x_r, qs = [self.x_r])
        self._wheel21_disk = Disk(I=S.Half*self.m*self.R**2, pos1=self.x_r/self.R, qs=[self.x_r])
        self._wheel22 = MaterialPoint(self.m, self.x_r, qs = [self.x_r])
        self._wheel22_disk = Disk(I=S.Half*self.m*self.R**2, pos1=self.x_r/self.R, qs=[self.x_r])
        self._trolley2 = MaterialPoint(self.m_2, self.x_r, qs=[self.x_r])
        self._spring2 = Spring(2*self.k_r, pos1 = self.x_r, qs=[self.x_r])
        self._damper2 = Damper(self.c_r, pos1 = self.x_r, qs=[self.x_r])

        components['trolley_1'] = self._trolley1
        components['spring_1_top'] = self._spring1_top
        components['spring_1_bottom'] = self._spring1_bottom
        components['damper_1'] = self._damper1
        components['spring_12'] = self._spring12
        components['wheel12'] = self._wheel12
        components['wheel11'] = self._wheel11
        components['wheel11_disk'] = self._wheel11_disk
        components['wheel12_disk'] = self._wheel12_disk
        components['damper_12'] = self._damper12
        components['trolley_2'] = self._trolley2
        components['spring_2'] = self._spring2
        components['damper_2'] = self._damper2
        components['wheel22'] = self._wheel22
        components['wheel21'] = self._wheel21
        components['wheel22_disk'] = self._wheel22_disk
        components['wheel21_disk'] = self._wheel21_disk
        components['force'] = self._force
        
        return components

    def get_default_data(self):

        m0, k0, lam, Omega0, F0 = symbols('m_0 k_0 lambda Omega_0 F_0', positive=True)

        default_data_dict = {
            self.k_l: [S.One * self.k_c],
            self.k_r: [S.One * self.k_c],
            self.c_r: [S.One * lam * self.k_r],
            self.c_l: [S.One * lam * self.k_l],
            self.c_c: [S.One * lam * self.k_c],

            self.m_2: [S.One * self.m],
            self.m_1: [S.One* self.m],
            
#             self.Omega: [S.One * Omega0],
            self.F: [S.One * F0 * no for no in range(5,25)],
            self.m: [S.One * m0 * no for no in range(1,5)],
            self.m_2: [S.One * m0 * no for no in range(1,5)],
            self.m_1: [S.One * m0 * no for no in range(1,5)],
            
            self.k_c: [S.One * k0 * no for no in range(60, 90)],
            self.k_l: [S.One * k0 * no for no in range(60, 90)],
            self.k_r: [S.One * k0 * no for no in range(60, 90)],            
#             self.c_r: [S.One * lam * self.k_c],
#             self.c_l: [S.One * lam * self.k_c],
#             self.c_c: [S.One * lam * self.k_c],
            
        }

        return default_data_dict

    def get_numerical_data(self):

        default_data_dict = {
            self.m_1 : [100],
            self.m_2 : [200],
            self.k_l : [50],
            self.k_r : [50],
            self.k_c : [50],
            self.c_l : [100],
            self.c_r : [100],
            self.c_c : [100],
            self.Omega : [0.5 * 3.14, 1 * 3.14, 2 * 3.14, 4 * 3.14],
            self.F : [0.5 * 100, 1 * 100, 2 * 100, 4 * 100]
        }

        return default_data_dict

    def get_random_parameters(self):

        default_data_dict = self.get_default_data()

        parameters_dict = {
            key: random.choice(items_list)
            for key, items_list in default_data_dict.items()
        }

        parameters_dict[self.k_r] = parameters_dict[self.k_l]
        parameters_dict[self.m_1] = parameters_dict[self.m]
        parameters_dict[self.m_2] = parameters_dict[self.m]

        return parameters_dict

    def unit_dict(self):
        units_dict = {
        self.m: ureg.kilogram,  # Mass of the trolley
        self.Omega: ureg.radian / ureg.second,  # Frequency of the force (rad/s)
        self.ivar: ureg.second,  # Time (seconds)
        self.k_l: ureg.newton / ureg.meter,  # Spring constant (units depend on spring type)
        self.k_r: ureg.newton / ureg.meter,  # Spring constant (units depend on spring type)
        self.k_c: ureg.newton / ureg.meter,  # Spring constant (units depend on spring type)
        self.x_l: ureg.meter,
        self.x_r: ureg.meter,
        self.x_l.diff(self.ivar): ureg.meter/ureg.second,
        self.x_r.diff(self.ivar): ureg.meter/ureg.second,
        self.x_l.diff(self.ivar,2): ureg.meter/ureg.second/ureg.second,
        self.x_r.diff(self.ivar,2): ureg.meter/ureg.second/ureg.second,
        self.c_l: ureg.newton*ureg.second / ureg.meter,  # Damping coefficient
        self.c_r: ureg.newton*ureg.second / ureg.meter,  # Damping coefficient
        self.c_c: ureg.newton*ureg.second / ureg.meter,  # Damping coefficient
        self.F: ureg.newton,  # Force amplitude
        }

        return units_dict


### Nieliniowe
class ForcedTrolleysWithNonLinearSprings(NonlinearComposedSystem):
    
    scheme_name = 'nonlin_trolley.PNG'
    real_name = 'nonlin_trolley_real.PNG'

    m_1=Symbol('m_1', positive=True)
    m_2=Symbol('m_2', positive=True)
    k=Symbol('k', positive=True)
    Omega=Symbol('Omega', positive=True)
    F=Symbol('F', positive=True)
    G=Symbol('G', positive=True)
    mu = Symbol('\\mu', positive=True)
    x_1=dynamicsymbols('x_1')
    x_2=dynamicsymbols('x_2')
   
    def __init__(self,
                 m_1=None,
                 m_2=None,
                 k=None,
                 Omega=None,
                 F=None,
                 G=None,
                 mu=None,
                 x_1=None,
                 x_2=None,
                 ivar=Symbol('t'),
                 **kwargs):

        if m_1 is not None: self.m_1 = m_1
        if m_2 is not None: self.m_2 = m_2
        if k is not None: self.k= k
        if mu is not None: self.mu= mu
        if Omega is not None: self.Omega = Omega
        if F is not None: self.F = F
        if G is not None: self.G = G
        if x_1 is not None: self.x_1 = x_1
        if x_2 is not None: self.x_2 = x_2

        self.qs = [self.x_1, self.x_2]
        self._init_from_components(**kwargs)

    @property
    def components(self):

        components = {}

        self._trolley1 = MaterialPoint(self.m_1, self.x_1, qs=[self.x_1])
        self._spring1 = Spring(self.k*(self.mu*self.x_1**2), pos1 = self.x_1, qs=[self.x_1])
        self._force = Force(self.F*sin(self.Omega*self.ivar), pos1 = self.x_1, qs=[self.x_1])

        self._spring12 = Spring(self.k*(self.mu*(self.x_1-self.x_2)**2), pos1 = self.x_1, pos2 = self.x_2, qs=[self.x_1, self.x_2])
        self._trolley2 = MaterialPoint(self.m_2, self.x_2, qs=[self.x_2])
        self._spring2 = Spring(self.k*(self.mu*self.x_2**2), pos1 = self.x_2, qs=[self.x_2])


        components['trolley_1'] = self._trolley1
        components['spring_1'] = self._spring1
        components['spring_12'] = self._spring12
        components['trolley_2'] = self._trolley2
        components['spring_2'] = self._spring2
        components['force'] = self._force
        
        return components
    
    def static_force(self):
        data=self._given_data
        ans=self.dynamic_force()
        free_coeff=ans.subs({cos(self.Omega*self.ivar):0, sin(self.Omega*self.ivar):0}).subs(data)
        return (free_coeff)
    
    def dynamic_force(self):
        amps = self._fodes_system.steady_solution.as_dict()
        data=self._given_data

        return (self.components['spring_1'].force().subs(amps)).subs(data).expand().doit()

    
class ForcedThreeTrolleysWithSprings(ComposedSystem): ### 7 ODE
    
    scheme_name = 'nonlin_trolley.PNG'
    real_name = 'nonlin_trolley_real.PNG'

    m_1=Symbol('m_1', positive=True)
    m_2=Symbol('m_2', positive=True)
    m_3=Symbol('m_3', positive=True)
    k=Symbol('k', positive=True)
    Omega=Symbol('Omega', positive=True)
    F=Symbol('F', positive=True)
    G=Symbol('G', positive=True)
    x_1=dynamicsymbols('x_1')
    x_2=dynamicsymbols('x_2')
    x_3=dynamicsymbols('x_3')
   
    def __init__(self,
                 m_1=None,
                 m_2=None,
                 m_3=None,
                 k=None,
                 Omega=None,
                 F=None,
                 G=None,
                 x_1=None,
                 x_2=None,
                 x_3=None,
                 ivar=Symbol('t'),
                 **kwargs):

        if m_1 is not None: self.m_1 = m_1
        if m_2 is not None: self.m_2 = m_2
        if m_3 is not None: self.m_3 = m_3
        if k is not None: self.k= k
        if Omega is not None: self.Omega = Omega
        if F is not None: self.F = F
        if G is not None: self.G = G
        if x_1 is not None: self.x_1 = x_1
        if x_2 is not None: self.x_2 = x_2
        if x_3 is not None: self.x_3 = x_3

        self.qs = [self.x_1, self.x_2, self.x_3]
        self._init_from_components(**kwargs)

    @property
    def components(self):

        components = {}

        self._trolley1 = MaterialPoint(self.m_1, self.x_1, qs=[self.x_1])
        self._spring1 = Spring(self.k, pos1 = self.x_1, qs=[self.x_1])
        self._force = Force(self.F*sin(self.Omega*self.ivar) + self.G, pos1 = self.x_1, qs=[self.x_1])
        
        self._spring12 = Spring(self.k, pos1 = self.x_1, pos2 = self.x_2, qs=[self.x_1, self.x_2])
        self._trolley2 = MaterialPoint(self.m_2, self.x_2, qs=[self.x_2])
        self._spring2 = Spring(self.k, pos1 = self.x_2, qs=[self.x_2])
        
        self._spring23 = Spring(self.k, pos1 = self.x_2, pos2 = self.x_3, qs=[self.x_2, self.x_3])
        self._trolley3 = MaterialPoint(self.m_3, self.x_3, qs=[self.x_3])
        self._spring3 = Spring(self.k, pos1 = self.x_3, qs=[self.x_3])

        components['trolley_1'] = self._trolley1
        components['spring_1'] = self._spring1
        components['spring_12'] = self._spring12
        components['trolley_2'] = self._trolley2
        components['spring_2'] = self._spring2
        components['spring_23'] = self._spring23
        components['trolley_3'] = self._trolley3
        components['spring_3'] = self._spring3
        components['force'] = self._force
        
        return components

    def get_numerical_data(self):

        default_data_dict = {
            self.m_1 : [100],
            self.m_2 : [200],
            self.m_3 : [300],
            self.k : [50],
            self.Omega : [0.5 * 3.14, 1 * 3.14, 2 * 3.14, 4 * 3.14],
            self.F : [0.5 * 100, 1 * 100, 2 * 100, 4 * 100]
        }
        default_data_dict.update({self.G: [4*default_data_dict[self.F][0]*cos(0.5*default_data_dict[self.Omega][0]*self.ivar) , default_data_dict[self.F][0]*cos(0.75*default_data_dict[self.Omega][0]*self.ivar)**2 , 1.5*default_data_dict[self.F][0]*2*cos(1.25*default_data_dict[self.Omega][0]*self.ivar) , 3*default_data_dict[self.F][0]*cos(2*default_data_dict[self.Omega][0]*self.ivar)**2]})

        return default_data_dict
    


class ForcedDisconnectedTrolleysWithSprings(ForcedThreeTrolleysWithSprings): ### 5 ODE

    @property
    def components(self):

        components = {}

        self._trolley1 = MaterialPoint(self.m_1, self.x_1, qs=[self.x_1])
        self._spring1 = Spring(self.k, pos1 = self.x_1, qs=[self.x_1])
        self._force = Force(self.F*sin(self.Omega*self.ivar), pos1 = self.x_1, qs=[self.x_1])
        
        self._spring12 = Spring(self.k, pos1 = self.x_1, qs=[self.x_1])
        #self._trolley2 = MaterialPoint(self.m_2, self.x_2, qs=[self.x_2])
        self._spring23 = Spring(self.k, pos1= -self.x_3, qs=[self.x_3])
        
        self._trolley3 = MaterialPoint(self.m_3, self.x_3, qs=[self.x_3])
        self._spring3 = Spring(self.k, pos1 = self.x_3, qs=[self.x_3])

        components['trolley_1'] = self._trolley1
        components['spring_1'] = self._spring1
        components['spring_12'] = self._spring12
        #components['trolley_2'] = self._trolley2
        components['spring_23'] = self._spring23
        components['trolley_3'] = self._trolley3
        components['spring_3'] = self._spring3
        components['force'] = self._force
        
        return components    
    


class ForcedDampedThreeTrolleysWithSprings(ComposedSystem): ### 8 ODE
    
    scheme_name = 'nonlin_trolley.PNG'
    real_name = 'nonlin_trolley_real.PNG'

    m_1=Symbol('m_1', positive=True)
    m_2=Symbol('m_2', positive=True)
    m_3=Symbol('m_3', positive=True)
    k=Symbol('k', positive=True)
    c=Symbol('c', positive=True)
    Omega=Symbol('Omega', positive=True)
    F=Symbol('F', positive=True)
    G=Symbol('G', positive=True) ## Siła
    x_1=dynamicsymbols('x_1')
    x_2=dynamicsymbols('x_2')
    x_3=dynamicsymbols('x_3')
   
    def __init__(self,
                 m_1=None,
                 m_2=None,
                 m_3=None,
                 k=None,
                 c=None,
                 Omega=None,
                 F=None,
                 G=None,
                 x_1=None,
                 x_2=None,
                 x_3=None,
                 ivar=Symbol('t'),
                 **kwargs):

        if m_1 is not None: self.m_1 = m_1
        if m_2 is not None: self.m_2 = m_2
        if m_3 is not None: self.m_3 = m_3
        if k is not None: self.k= k
        if c is not None: self.c= c
        if Omega is not None: self.Omega = Omega
        if F is not None: self.F = F
        if G is not None: self.G = G
        if x_1 is not None: self.x_1 = x_1
        if x_2 is not None: self.x_2 = x_2
        if x_3 is not None: self.x_3 = x_3

        self.qs = [self.x_1, self.x_2, self.x_3]
        self._init_from_components(**kwargs)

    @property
    def components(self):

        components = {}

        self._trolley1 = MaterialPoint(self.m_1, self.x_1, qs=[self.x_1])
        self._spring1 = Spring(self.k, pos1 = self.x_1, qs=[self.x_1])
        self._damper1 = Damper(self.c, pos1 = self.x_1, qs=[self.x_1])
        self._force = Force(self.F*sin(self.Omega*self.ivar) + self.G, pos1 = self.x_1, qs=[self.x_1])
        
        self._spring12 = Spring(self.k, pos1 = self.x_1, pos2 = self.x_2, qs=[self.x_1, self.x_2])
        self._damper12 = Damper(self.c, pos1 = self.x_1, pos2 = self.x_2, qs=[self.x_1, self.x_2])
        
        self._trolley2 = MaterialPoint(self.m_2, self.x_2, qs=[self.x_2])
        
        self._spring23 = Spring(self.k, pos1 = self.x_2, pos2 = self.x_3, qs=[self.x_2, self.x_3])
        self._damper23 = Damper(self.c, pos1 = self.x_2, pos2 = self.x_3, qs=[self.x_2, self.x_3])
        
        self._trolley3 = MaterialPoint(self.m_3, self.x_3, qs=[self.x_3])
        self._spring3 = Spring(self.k, pos1 = self.x_3, qs=[self.x_3])
        self._damper3 = Damper(self.c, pos1 = self.x_3, qs=[self.x_3])

        components['trolley_1'] = self._trolley1
        components['spring_1'] = self._spring1
        components['damper_1'] = self._damper1
        components['spring_12'] = self._spring12
        components['damper_12'] = self._damper12
        components['trolley_2'] = self._trolley2
        components['spring_23'] = self._spring23
        components['damper_23'] = self._damper23
        components['trolley_3'] = self._trolley3
        components['spring_3'] = self._spring3
        components['damper_3'] = self._damper3
        components['force'] = self._force
        
        return components
    
    def get_numerical_data(self):

        default_data_dict = {
            self.m_1 : [100],
            self.m_2 : [200],
            self.m_3 : [300],
            self.k : [50],
            self.c : [100],
            self.Omega : [0.5 * 3.14, 1 * 3.14, 2 * 3.14, 4 * 3.14],
            self.F : [0.5 * 100, 1 * 100, 2 * 100, 4 * 100]
        }
        default_data_dict.update({self.G: [4*default_data_dict[self.F][0]*cos(0.5*default_data_dict[self.Omega][0]*self.ivar) , default_data_dict[self.F][0]*cos(0.75*default_data_dict[self.Omega][0]*self.ivar)**2 , 1.5*default_data_dict[self.F][0]*2*cos(1.25*default_data_dict[self.Omega][0]*self.ivar) , 3*default_data_dict[self.F][0]*cos(2*default_data_dict[self.Omega][0]*self.ivar)**2]})

        return default_data_dict



class ForcedDampedDisconnectedTrolleysWithSprings(ForcedDampedThreeTrolleysWithSprings): ### 6 ODE

    @property
    def components(self):

        components = {}

        self._trolley1 = MaterialPoint(self.m_1, self.x_1, qs=[self.x_1])
        self._spring1 = Spring(self.k, pos1 = self.x_1, qs=[self.x_1])
        self._damper1 = Damper(self.c, pos1 = self.x_1, qs=[self.x_1])
        self._force = Force(self.F*sin(self.Omega*self.ivar) + self.G, pos1 = self.x_1, qs=[self.x_1])
        
        self._spring12 = Spring(self.k, pos1 = self.x_1, qs=[self.x_1])
        self._damper12 = Damper(self.c, pos1 = self.x_1, qs=[self.x_1])
        
        #self._trolley2 = MaterialPoint(self.m_2, self.x_2, qs=[self.x_2])
        
        self._spring23 = Spring(self.k, pos1 = -self.x_3, qs=[self.x_3])
        self._damper23 = Damper(self.c, pos1 = -self.x_3, qs=[self.x_3])
        
        self._trolley3 = MaterialPoint(self.m_3, self.x_3, qs=[self.x_3])
        self._spring3 = Spring(self.k, pos1 = self.x_3, qs=[self.x_3])
        self._damper3 = Damper(self.c, pos1 = self.x_3, qs=[self.x_3])

        components['trolley_1'] = self._trolley1
        components['spring_1'] = self._spring1
        components['damper_1'] = self._damper1
        components['spring_12'] = self._spring12
        components['damper_12'] = self._damper12
        #components['trolley_2'] = self._trolley2
        components['spring_23'] = self._spring23
        components['damper_23'] = self._damper23
        components['trolley_3'] = self._trolley3
        components['spring_3'] = self._spring3
        components['damper_3'] = self._damper3
        components['force'] = self._force
        
        return components



class DDoFTwoNonLinearTrolleys(NonlinearComposedSystem):
    
    scheme_name = 'ForcedTwoNonLinearTrolleys.png'
    real_name = 'tension_leg_platform.png'

    g=Symbol('g', positive=True)
    m1=Symbol('m_1', positive=True)
    m2=Symbol('m_2', positive=True)
    k1=Symbol('k_1', positive=True)
    k2=Symbol('k_2', positive=True)
    k3=Symbol('k_3', positive=True)
    d=Symbol('d', positive=True)
    l_0=Symbol('l_0', positive=True)
    ivar=Symbol('t')
    x1=dynamicsymbols('x1')
    x2=dynamicsymbols('x2')
    x=dynamicsymbols('x')
    qs=dynamicsymbols('x1, x2')
    F1=Symbol('F_1', positive=True)
    F2=Symbol('F_2', positive=True)
    Omega=Symbol('Omega', positive=True)
   
    def __init__(self,
                 g=None,
                 m1=None,
                 m2=None,
                 k1=None,
                 k2=None,
                 k3=None,
                 d=None,
                 l_0=None,
                 ivar=Symbol('t'),
                 x1=None,
                 x2=None,
                 x=None,
                 qs=None,
                 F1=None,
                 F2=None,
                 Omega=None,
                 **kwargs):

        if m1 is not None: self.m1 = m1
        if m2 is not None: self.m2 = m2
        if k1 is not None: self.k1= k1
        if k2 is not None: self.k2= k2
        if k3 is not None: self.k3= k3
        if d is not None: self.d = d
        if l_0 is not None: self.l_0 = l_0
        if Omega is not None: self.Omega = Omega
        if F1 is not None: self.F1 = F1
        if F2 is not None: self.F2 = F2
        if x is not None: self.x = x
        if x1 is not None: self.x1 = x1
        if x2 is not None: self.x2 = x2
        if qs is not None: self.qs = qs
        if g is not None: self.g = g
        self.ivar=ivar

        self._init_from_components(**kwargs)

    @property
    def components(self):

        components = {}

        self._trolley1 = MaterialPoint(self.m1, self.x1, qs=self.qs)
        self._spring1 = Spring(self.k1, pos1=(sqrt(self.x1**2 + self.d**2) - self.l_0), qs=self.qs)
        self._trolley2 = MaterialPoint(self.m2, self.x2, qs=self.qs)
        self._spring2 = Spring(self.k2, pos1=(sqrt(self.x2**2 + self.d**2) - self.l_0), qs=self.qs)
        self._spring12 = Spring(self.k3, self.x1, self.x2,qs=self.qs)
        self._force1 = Force(self.F1*sin(self.Omega*self.ivar), self.x1, qs=self.qs)
        self._force2 = Force(self.F2*sin(self.Omega*self.ivar), self.x2, qs=self.qs)

        components['_trolley1'] = self._trolley1
        components['_trolley2'] = self._trolley2
        components['_spring1'] = self._spring1
        components['_spring2'] = self._spring2
        components['_spring12'] = self._spring12
        components['_force1'] = self._force1
        components['_force2'] = self._force2
        
        return components

    def get_default_data(self):

        m0, k0, l0, F0 = symbols('m_0 k_0 l_0 F_0', positive=True)

        default_data_dict = {
            self.m1: [S.One * m0 * no for no in range(20,50)],
            self.m2: [S.One * m0 * no for no in range(20,50)],
            self.d: [S.One * l0 * no for no in range(2,5)],
            self.k1: [S.One * k0 * no for no in range(10,20)],
            self.k2: [S.One * k0 * no for no in range(10,20)],
            self.k3: [S.One * k0 * no for no in range(10,20)],
            self.F1: [S.One * F0 * no for no in range(5,15)],
            self.F2: [S.One * F0 * no for no in range(5,15)],
            self.x1: [self.x, 0],
            self.x2: [self.x, S.Zero],

        }

        return default_data_dict

    def get_random_parameters(self):

        default_data_dict = self.get_default_data()

        parameters_dict = {
            key: random.choice(items_list)
            for key, items_list in default_data_dict.items()
        }

        if parameters_dict[self.x1] == S.Zero:
            parameters_dict[self.x2] = self.x

        return parameters_dict

    def symbols_description(self):
        self.sym_desc_dict = {
            self.m1: r'Trolley Mass',
            self.m2: r'Trolley Mass',
            self.k1: 'Spring Stiffness',
            self.k2: 'Spring Stiffness',
            self.k3: 'Spring Stiffness',
            self.d: r'length',
            self.l_0: r'length',
        }
        return self.sym_desc_dict

    def static_force(self):
        data=self._given_data
        ans=self.dynamic_force()
        free_coeff=ans.subs({cos(self.Omega*self.ivar):0, sin(self.Omega*self.ivar):0}).subs(data)
        return (free_coeff)

#     def dynamic_force(self):

#         data=self._given_data
#         amps = self._fodes_system.steady_solution.as_dict()

#         return (self.components['_spring12'].force().subs(amps)).subs(data).expand().doit()

    def dynamic_force(self):

        data=self._given_data
        amps = self.linearized().subs(data)._fodes_system.steady_solution.as_dict()


        return (self.components['_spring12'].force().subs(data).subs(amps)).expand().doit()



#issue 204 zrobione karolinka + bogus
class DoubleTrolleyWithNonlinearSprings(ComposedSystem):
    scheme_name = 'MDOF_Double_Trolley_With_Springs.PNG'
    real_name = 'sdof_nonlinspring_trolleys_real.PNG'

    R=Symbol('R', positive=True)
    m=Symbol('m', positive=True)
    m1=Symbol('m_1', positive=True)
    m2=Symbol('m_2', positive=True)
    k_l=Symbol('k_l', positive=True)
    k_cl=Symbol('k_cl', positive=True)
    k_c=Symbol('k_c', positive=True)
    k_cc=Symbol('k_cc', positive=True)
    k_r=Symbol('k_r', positive=True)
    k_cr=Symbol('k_cr', positive=True)
    mu=Symbol('mu', positive=True)
    x_l=dynamicsymbols('x_l')
    x_r=dynamicsymbols('x_r')
    x=dynamicsymbols('x')
    qs=dynamicsymbols('x_l x_r')
    ivar=Symbol('t')



    def __init__(self,
                 R=None,
                 m=None,
                 m1=None,
                 m2=None,
                 k_l=None,
                 k_cl=None,
                 k_c=None,
                 k_cc=None,
                 k_r=None,
                 k_cr=None,
                 mu=None,
                 x_l=None,
                 x_r=None,
                 x=None,
                 qs=None,
                 ivar=Symbol('t'),
                 **kwargs):

        if m is not None: self.m = m
        if m1 is not None: self.m1 = m1
        if m2 is not None: self.m2 = m2
        if k_l is not None: self.k_l = k_l
        if k_cl is not None: self.k_cl = k_cl
        if k_c is not None: self.k_c = k_c
        if k_cc is not None: self.k_cc = k_cc
        if k_r is not None: self.k_r = k_r
        if k_cr is not None: self.k_cr = k_cr
        if x_l is not None: self.x_l = x_l
        if x_r is not None: self.x_r = x_r
        if x is not None: self.x = x
        if R is not None: self.R = R
        if mu is not None: self.mu = mu
        if qs is not None: self.qs=qs

        self.ivar=ivar
        self._init_from_components(**kwargs)

    @property
    def components(self):

        components = {}

        self._trolley_1 = ForcedTrolleyWithThreeSprings(self.m1, self.k_l, self.k_cl*(1-self.mu*self.x_l**2), 0, z=self.x_l, qs=[self.x_l])
        self._three_springs=ThreeSprings(self.k_c, self.k_cc*(1-self.mu*(self.x_r-self.x_l)**2),pos1=self.x_l,pos2=self.x_r, qs=self.qs)
        self._rolling_disk1=RollingDisk(self.m,self.R,self.x_l/2,qs=[self.x_l])
        self._rolling_disk2=RollingDisk(self.m,self.R,self.x_l/2,qs=[self.x_l])
        self._trolley_2= ForcedTrolleyWithThreeSprings(self.m2, self.k_r, self.k_cr*(1-self.mu*self.x_r**2), 0, z=self.x_r, qs=[self.x_r])
        self._rolling_disk3=RollingDisk(self.m,self.R,self.x_r/2,qs=[self.x_r])
        self._rolling_disk4=RollingDisk(self.m,self.R,self.x_r/2,qs=[self.x_r])

        components['_trolley_1'] = self._trolley_1
        components['_three_springs'] = self._three_springs
        components['_trolley_2'] = self._trolley_2
        components['_rolling_disk1'] = self._rolling_disk1
        components['_rolling_disk2'] = self._rolling_disk2
        components['_rolling_disk3'] = self._rolling_disk3
        components['_rolling_disk4'] = self._rolling_disk4

        return components



#DONE(#206)
class DoubleTrolleyDifferentWheels(ComposedSystem):
    scheme_name = 'MDOFDoubleTrolleyDifferentWheels.PNG'
    real_name = 'DoubleTrolley_real.png'
    
    R=Symbol('R', positive=True)
    m_w1=Symbol('m_w1', positive=True)
    m_w2=Symbol('m_w2', positive=True)
    m1=Symbol('m_1', positive=True)
    m2=Symbol('m_2', positive=True)
    k_l=Symbol('k_l', positive=True)
    c_cl=Symbol('c_cl', positive=True)
    k_c=Symbol('k_c', positive=True)
    c_cc=Symbol('c_cc', positive=True)
    k_r=Symbol('k_r', positive=True)
    c_cr=Symbol('c_cr', positive=True)
    lam=Symbol('lambda', positive=True)
    x_l=dynamicsymbols('x_l')
    x_r=dynamicsymbols('x_r')

    qs=[x_l, x_r]
    
    def __init__(self,
                 R=None,
                 m_w1=None,
                 m_w2=None,
                 m1=None,
                 m2=None,
                 k_l=None,
                 c_cl=None,
                 k_c=None,
                 c_cc=None,
                 k_r=None,
                 c_cr=None,
                 lam=None,
                 x_l=None,
                 x_r=None,
                 qs=None,
                 ivar=Symbol('t'),
                 **kwargs):

        if x_l is not None: self.x_l = x_l
        if x_r is not None: self.x_r = x_r
        if m1 is not None: self.m1 = m1
        if m2 is not None: self.m2 = m2
        if m_w1 is not None: self.m_w1 = m_w1
        if m_w2 is not None: self.m_w2 = m_w2
        if k_l is not None: self.k_l = k_l
        if c_cl is not None: self.c_cl = c_cl
        if k_c is not None: self.k_c = k_c
        if c_cc is not None: self.c_cc = c_cc
        if k_r is not None: self.k_r = k_r
        if c_cr is not None: self.c_cr = c_cr
        if R is not None: self.R = R
        if lam is not None: self.lam = lam
        if qs is not None: self.qs=qs
        
        self.ivar=ivar
        self._init_from_components(**kwargs)

        
    @property
    def components(self):

        components = {}


        self._trolley_1 = MaterialPoint(self.m1, self.x_l, qs=self.qs)
        self._rolling_disk1=RollingDisk(self.m_w1,self.R,self.x_l/2, qs=self.qs)
        self._rolling_disk2=RollingDisk(self.m_w1,self.R,self.x_l/2, qs=self.qs)
        self._k_l_spring_1 = Spring(self.k_l, pos1=self.x_l,  qs=self.qs)
        self._k_l_spring_2 = Spring(self.k_l, pos1=self.x_l,  qs=self.qs)
        self._c_cl_damper = Damper(self.c_cl, pos1=self.x_l,  qs=self.qs)
        
        
        self._trolley_2 = MaterialPoint(self.m1, self.x_r, qs=self.qs)
        self._rolling_disk3=RollingDisk(self.m_w2,self.R,self.x_r/2, qs=self.qs)
        self._rolling_disk4=RollingDisk(self.m_w2,self.R,self.x_r/2, qs=self.qs)
        self._k_r_spring_1 = Spring(self.k_r, pos1=self.x_r,  qs=self.qs)
        self._k_r_spring_2 = Spring(self.k_r, pos1=self.x_r,  qs=self.qs)
        self._c_cr_damper = Damper(self.c_cr, pos1=self.x_r,  qs=self.qs)
        
        self._c_cc_damper = Damper(self.c_cc, pos1=self.x_l, pos2=self.x_r, qs=self.qs)
        self._k_c_spring_1 = Spring(self.k_c, pos1=self.x_l, pos2=self.x_r, qs=self.qs)
        self._k_c_spring_2 = Spring(self.k_c, pos1=self.x_l, pos2=self.x_r, qs=self.qs)


        components['_trolley_1'] = self._trolley_1
        
        components['_trolley_2'] = self._trolley_2
        components['_rolling_disk1'] = self._rolling_disk1
        components['_rolling_disk2'] = self._rolling_disk2
        
        components['_k_l_spring_1'] = self._k_l_spring_1
        components['_k_l_spring_2'] = self._k_l_spring_2
        components['_c_cl_damper'] = self._c_cl_damper
        

        components['_rolling_disk3'] = self._rolling_disk3
        components['_rolling_disk4'] = self._rolling_disk4

        components['_k_r_spring_1'] = self._k_r_spring_1
        components['_k_r_spring_2'] = self._k_r_spring_2
        components['_c_cr_damper'] = self._c_cr_damper
        
        components['_k_c_spring_1'] = self._k_c_spring_1
        components['_k_c_spring_2'] = self._k_c_spring_2
        components['_c_cc_damper'] = self._c_cc_damper
        
        
        return components

    def get_default_data(self):

        m0, k0, l0, lam = symbols('m k l_0 lambda', positive=True)

        default_data_dict = {
            self.m1: [1 * m0, 2 * m0, S.Half * m0, S.Half**2 *  m0, 2**2 * m0],
            self.m2: [1 * m0, 2 * m0, S.Half * m0, S.Half**2 *  m0, 2**2 * m0],
            self.m_w1: [1 * m0, 2 * m0, S.Half * m0, S.Half**2 *  m0, 2**2 * m0],
            self.m_w2: [1 * m0, 2 * m0, S.Half * m0, S.Half**2 *  m0, 2**2 * m0],
            self.k_l: [1 * k0, 2 * k0, S.Half * k0, 4 * k0, S.Half**2 * k0],
            self.k_c: [1 * k0, 2 * k0, S.Half * k0, 4 * k0, S.Half**2 * k0],
            self.k_r: [1 * k0, 2 * k0, S.Half * k0, 4 * k0, S.Half**2 * k0],

            self.c_cr: [lam *  k0,lam * 2 * k0,lam * S.Half * k0,lam * 4 * k0,lam * S.Half**2 * k0],
            self.c_cc: [lam *  k0,lam * 2 * k0,lam * S.Half * k0,lam * 4 * k0,lam * S.Half**2 * k0],
            self.c_cl: [lam *  k0,lam * 2 * k0,lam * S.Half * k0,lam * 4 * k0,lam * S.Half**2 * k0],
            
            
            self.lam:[0],


        }

        return default_data_dict

    def get_numerical_data(self):
        
        numerical_data_dict = {
            self.m1: [1],
            self.m2: [1],
            self.m_w1: [0.1],
            self.m_w2: [0.5],
            self.k_l: [50],
            self.k_c: [70],
            self.k_r: [50],

            self.c_cr: [0.3],
            self.c_cc: [0.1],
            self.c_cl: [0.3],
            self.R: [0.05],
        }

        return numerical_data_dict

    
    def symbols_description(self):
        self.sym_desc_dict = {
            self.m1: r'Trolley Mass',
            self.m2: r'Trolley Mass',
            self.m_w1: r'Wheels 1 Mass',
            self.m_w2: r'Wheels 2 Mass',
            self.k_l: r'Spring Stiffness',
            self.k_c: r'Spring Stiffness',
            self.k_r: r'Spring Stiffness',

            self.c_cr: r'Damper damping',
            self.c_cc: r'Damper damping',
            self.c_cl: r'Damper damping',
            self.R: r'Wheels radius',

        }
        return self.sym_desc_dict
    
    def unit_dict(self):

        unit_dict = {

            self.ivar: ureg.second,

            self.m1: ureg.kilogram,
            self.m2: ureg.kilogram,
            self.m_w1: ureg.kilogram,
            self.m_w2: ureg.kilogram,
            self.k_l: ureg.newton/ureg.meter,
            self.k_c: ureg.newton/ureg.meter,
            self.k_r: ureg.newton/ureg.meter,

            self.c_cr: ureg.newton/(ureg.meter/ureg.second),
            self.c_cc: ureg.newton/(ureg.meter/ureg.second),
            self.c_cl: ureg.newton/(ureg.meter/ureg.second),
            self.x_l: ureg.meter,
            self.x_r: ureg.meter,
            self.x_l.diff(self.ivar): ureg.meter/ureg.second,
            self.x_r.diff(self.ivar): ureg.meter/ureg.second,
            self.x_l.diff(self.ivar,2): ureg.meter/ureg.second/ureg.second,
            self.x_r.diff(self.ivar,2): ureg.meter/ureg.second/ureg.second,
            self.R: ureg.meter,
        }
        return unit_dict

    
#DONE(#205)
class DampedTrolleysWithSprings(DoubleTrolleyDifferentWheels):
#    scheme_name = 'MDOFDoubleTrolleyDifferentWheels.png'
    scheme_name = 'MDOFDoubleTrolley.png'
    real_name = 'two_damped_trolleys.png'

    R=Symbol('R', positive=True)
    m=Symbol('m', positive=True)
    m1=Symbol('m_1', positive=True)
    m2=Symbol('m_2', positive=True)
    k_l=Symbol('k_l', positive=True)
    c_cl=Symbol('c_cl', positive=True)
    k_c=Symbol('k_c', positive=True)
    c_cc=Symbol('c_cc', positive=True)
    k_r=Symbol('k_r', positive=True)
    c_cr=Symbol('c_cr', positive=True)
    lam=Symbol('lambda', positive=True)
    x_l=dynamicsymbols('x_l')
    x_r=dynamicsymbols('x_r')

    qs=[x_l, x_r]

    def __init__(self,
                 R=None,
                 m1=None,
                 m2=None,
                 m=None,
                 k_l=None,
                 k_r=None,
                 k_c=None,
                 c_cl=None,
                 c_cr=None,
                 c_cc=None,
                 x_l=None,
                 x_r=None,
                 lam=None,
                 qs=None,
                 ivar=Symbol('t'),
                 **kwargs):

        if x_l is not None: self.x_l = x_l
        if x_r is not None: self.x_r = x_r
        if m1 is not None: self.m1 = m1
        if m2 is not None: self.m2 = m2
        if m is not None: self.m = m
        if k_l is not None: self.k_l = k_l
        if c_cl is not None: self.c_cl = c_cl
        if k_c is not None: self.k_c = k_c
        if c_cc is not None: self.c_cc = c_cc
        if k_r is not None: self.k_r = k_r
        if c_cr is not None: self.c_cr = c_cr
        if R is not None: self.R = R
        if lam is not None: self.lam = lam
        if qs is not None: self.qs=qs

        self.ivar=ivar
        self._init_from_components(**kwargs)

    @cached_property
    def components(self):

        components = {}

        self._system = DoubleTrolleyDifferentWheels(R=self.R,
                 m_w1=self.m,
                 m_w2=self.m,
                 m1=self.m1,
                 m2=self.m2,
                 k_l=self.k_l,
                 c_cl=self.c_cl,
                 k_c=self.k_c,
                 c_cc=self.c_cc,
                 k_r=self.k_r,
                 c_cr=self.c_cr,
                 lam=self.lam,
                 x_l=self.x_l,
                 x_r=self.x_r,
                 qs=self.qs,
                 ivar=self.ivar
                 )

        components['_system'] = self._system
        return components

    def get_default_data(self):

        m0, k0, l0, lam = symbols('m k l_0 lambda', positive=True)

        default_data_dict = {
            self.m1: [S.Half * m0, 1 * m0, 2 * m0, 4 * m0, S.Half**2 * m0],
            self.m2: [S.Half * m0, 1 * m0, 2 * m0, 4 * m0, S.Half**2 * m0],
            self.m: [S.Half * m0, 1 * m0, 2 * m0, 4 * m0, S.Half**2 * m0],

            self.k_l: [1 * k0, 2 * k0, S.Half * k0, 4 * k0, S.Half**2 * k0],
            self.k_c: [1 * k0, 2 * k0, S.Half * k0, 4 * k0, S.Half**2 * k0],
            self.k_r: [1 * k0, 2 * k0, S.Half * k0, 4 * k0, S.Half**2 * k0],

            self.c_cr: [lam *  k0,lam * 2 * k0,lam * S.Half * k0,lam * 4 * k0,lam * S.Half**2 * k0],
            self.c_cc: [lam *  k0,lam * 2 * k0,lam * S.Half * k0,lam * 4 * k0,lam * S.Half**2 * k0],
            self.c_cl: [lam *  k0,lam * 2 * k0,lam * S.Half * k0,lam * 4 * k0,lam * S.Half**2 * k0],

        }

        return default_data_dict


    def get_numerical_data(self):

        numerical_data_dict = {
            self.m1: [1],
            self.m2: [1],
            self.m: [0.1],
            self.k_l: [50],
            self.k_c: [70],
            self.k_r: [100],

            self.c_cr: [3.5],
            self.c_cc: [4.0],
            self.c_cl: [3.0],
            self.R: [0.05],
        }

        return numerical_data_dict

    
    def symbols_description(self):
        self.sym_desc_dict = {
            self.m1: r'Trolley Mass',
            self.m2: r'Trolley Mass',
            self.m: r'Wheels Mass',
            self.k_l: r'Spring Stiffness',
            self.k_c: r'Spring Stiffness',
            self.k_r: r'Spring Stiffness',

            self.c_cr: r'Damper damping',
            self.c_cc: r'Damper damping',
            self.c_cl: r'Damper damping',
            self.R: r'Wheels radius',
        }
        return self.sym_desc_dict
    
    def unit_dict(self):

        unit_dict = {

            self.ivar: ureg.second,

            
            self.m1: ureg.kilogram,
            self.m2: ureg.kilogram,
            self.m: ureg.kilogram,
            self.k_l: ureg.newton/ureg.meter,
            self.k_c: ureg.newton/ureg.meter,
            self.k_r: ureg.newton/ureg.meter,

            self.c_cr: ureg.newton/(ureg.meter/ureg.second),
            self.c_cc: ureg.newton/(ureg.meter/ureg.second),
            self.c_cl: ureg.newton/(ureg.meter/ureg.second),
            self.x_l: ureg.meter,
            self.x_r: ureg.meter,
            self.x_l.diff(self.ivar): ureg.meter/ureg.second,
            self.x_r.diff(self.ivar): ureg.meter/ureg.second,
            self.x_l.diff(self.ivar,2): ureg.meter/ureg.second/ureg.second,
            self.x_r.diff(self.ivar,2): ureg.meter/ureg.second/ureg.second,
            self.R: ureg.meter,
        }
        return unit_dict


# ISSUE 211 DONE karolina i bogus
# SAME AS ISUUE 207
class ForcedTrolleysWithSprings(ComposedSystem):

    scheme_name = 'mdof_three_trolleys.PNG'
    real_name = 'three_carriages.PNG'

    m1=Symbol('m_1', positive=True)
    m2=Symbol('m_2', positive=True)
    m3=Symbol('m_3', positive=True)
    k_l=Symbol('k_l', positive=True)
    k_cl=Symbol('k_cl', positive=True)
    k_12=Symbol('k_12', positive=True)
    k_c12=Symbol('k_c12', positive=True)
    k_23=Symbol('k_23', positive=True)
    k_c23=Symbol('k_c23', positive=True)
    k_r=Symbol('k_r', positive=True)
    k_cr=Symbol('k_cr', positive=True)
    F=Symbol('F', positive=True)
    F_1=Symbol('F_1', positive=True)
    F_2=Symbol('F_2', positive=True)
    Omega=Symbol('Omega', positive=True)

    x_l=dynamicsymbols('x_l')
    x_c=dynamicsymbols('x_c')
    x_r=dynamicsymbols('x_r')

    qs=dynamicsymbols('x_l x_c x_r')
    ivar=Symbol('t')

    def __init__(self,
                 m1=None,
                 m2=None,
                 m3=None,
                 k_l=None,
                 k_cl=None,
                 k_12=None,
                 k_c12=None,
                 k_23=None,
                 k_c23=None,
                 k_r=None,
                 k_cr=None,
                 x_l=None,
                 x_c=None,
                 x_r=None,
                 F=None,
                 F_1=None,
                 F_2=None,
                 Omega=None,
                 qs=None,
                 ivar=Symbol('t'),
                 **kwargs):

        if m1 is not None: self.m1 = m1
        if m2 is not None: self.m2 = m2
        if m3 is not None: self.m3 = m3
        if k_l is not None: self.k_l = k_l
        if k_cl is not None: self.k_cl = k_cl
        if k_12 is not None: self.k_12 = k_12
        if k_c12 is not None: self.k_c12 = k_c12
        if k_23 is not None: self.k_23 = k_23
        if k_c23 is not None: self.k_c23 = k_c23
        if k_r is not None: self.k_r = k_r
        if k_cr is not None: self.k_cr = k_cr
        if x_l is not None: self.x_l = x_l
        if x_c is not None: self.x_c = x_c
        if x_r is not None: self.x_r = x_r
        if F is not None: self.F = F
        if F_1 is not None: self.F_1 = F_1
        if F_2 is not None: self.F_2 = F_2
        if Omega is not None: self.Omega = Omega


        if qs is not None: self.qs =qs

        self.ivar=ivar

        self._init_from_components(**kwargs)



    @cached_property
    def components(self):

            components = {}
            F_1=-self.F * cos(self.Omega * self.ivar)
            F_2=-2 * self.F * cos(self.Omega * self.ivar)

            self._trolley_1 = ForcedTrolleyWithThreeSprings(self.m1, self.k_l, self.k_cl, F_1, z=self.x_l, qs=[self.x_l] )

            self._trolley_2 = MaterialPoint(self.m2, pos1=self.x_c, qs=self.qs)
            self._springs1=ThreeSprings(self.k_12, self.k_c12, pos1=self.x_l, pos2=self.x_c, qs=self.qs)
            self._springs2=ThreeSprings(self.k_23, self.k_c23, pos1=self.x_c, pos2=self.x_r, qs=self.qs)

            self._trolley_3 = ForcedTrolleyWithThreeSprings(self.m3, self.k_r, self.k_cr, F_2, z=self.x_r, qs=[self.x_r])


            components['_trolley_1'] = self._trolley_1
        
            components['_trolley_2'] = self._trolley_2
            components['_springs1'] = self._springs1
            components['_springs2'] = self._springs2
        
            components['_trolley_3'] = self._trolley_3
        
            return components
    def get_default_data(self):

        m0, k0, l0, lam = symbols('m k l_0 lambda', positive=True)

        default_data_dict = {
            self.m1: [S.Half * m0, 1 * m0, 2 * m0, 4 * m0, S.Half**2 * m0],
            self.m2: [S.Half * m0, 1 * m0, 2 * m0, 4 * m0, S.Half**2 * m0],
            self.m3: [S.Half * m0, 1 * m0, 2 * m0, 4 * m0, S.Half**2 * m0],
            
            self.k_l: [1 * k0, 2 * k0, S.Half * k0, 4 * k0, S.Half**2 * k0],
            self.k_cl: [1 * k0, 2 * k0, S.Half * k0, 4 * k0, S.Half**2 * k0],
            
            self.k_23: [1 * k0, 2 * k0, S.Half * k0, 4 * k0, S.Half**2 * k0],
            self.k_c23: [1 * k0, 2 * k0, S.Half * k0, 4 * k0, S.Half**2 * k0],
            
            self.k_12: [1 * k0, 2 * k0, S.Half * k0, 4 * k0, S.Half**2 * k0],
            self.k_c12: [1 * k0, 2 * k0, S.Half * k0, 4 * k0, S.Half**2 * k0],
            
            self.k_r: [1 * k0, 2 * k0, S.Half * k0, 4 * k0, S.Half**2 * k0],
            self.k_cr: [1 * k0, 2 * k0, S.Half * k0, 4 * k0, S.Half**2 * k0],
        }

        return default_data_dict


    def get_numerical_data(self):
        
        numerical_data_dict = {
            self.m1: [1],
            self.m2: [2],
            self.m3: [3],
            
            self.k_l: [5],
            self.k_cl: [8],
            
            self.k_23: [5],
            self.k_c23: [8],
            
            self.k_12: [5],
            self.k_c12: [8],
            
            self.k_r: [5],
            self.k_cr: [8],
            self.F: [5],
            
            self.Omega: [60],

        }

        return numerical_data_dict

    
    def symbols_description(self):
        self.sym_desc_dict = {
            self.m1: r'Trolley 1 Mass',
            self.m2: r'Trolley 2 Mass',
            self.m3: r'Trolley 3 Mass',

            self.k_l: r'Spring Stiffness',
            self.k_cl: r'Spring Stiffness',
            
            self.k_23: r'Spring Stiffness',
            self.k_c23: r'Spring Stiffness',
            
            self.k_12: r'Spring Stiffness',
            self.k_c12: r'Spring Stiffness',
            
            self.k_r: r'Spring Stiffness',
            self.k_cr: r'Spring Stiffness',


        }
        return self.sym_desc_dict
    
    def unit_dict(self):

        unit_dict = {

            self.ivar: ureg.second,

            
            self.m1: ureg.kilogram,
            self.m2: ureg.kilogram,
            self.m3: ureg.kilogram,

            self.k_l: ureg.newton/ureg.meter,
            self.k_cl: ureg.newton/ureg.meter,
            
            self.k_23: ureg.newton/ureg.meter,
            self.k_c23: ureg.newton/ureg.meter,
            
            self.k_12: ureg.newton/ureg.meter,
            self.k_c12: ureg.newton/ureg.meter,
            
            self.k_r: ureg.newton/ureg.meter,
            self.k_cr: ureg.newton/ureg.meter,
            

            self.x_l.diff(self.ivar): ureg.meter/ureg.second,
            self.x_r.diff(self.ivar): ureg.meter/ureg.second,
            self.x_c.diff(self.ivar): ureg.meter/ureg.second,
            self.x_l.diff(self.ivar,2): ureg.meter/ureg.second/ureg.second,
            self.x_r.diff(self.ivar,2): ureg.meter/ureg.second/ureg.second,
            self.x_c.diff(self.ivar,2): ureg.meter/ureg.second/ureg.second,

            self.F: ureg.newton,
            self.F_1: ureg.newton,
            self.F_2: ureg.newton,
            self.Omega: ureg.radian/ureg.second,
        }
        return unit_dict
    

#poprawione MDofTMD- ready to check
#klasa MDoFTMD została usunięta- zastępuje ją teraz TrolleyWithTMD
#TODO(#212 - check)







from sympy import *
from dynpy.solvers.linear import ODESystem
from dynpy.solvers.perturbational import MultiTimeScaleSolution
from dynpy.models.mechanics import ForcedSpringMassSystem
import numpy as np
import matplotlib.pyplot as plt

class LinearFirstOrder(ODESystem):

    @classmethod
    def from_reference_data(cls):
        t = Symbol('t')
        z = Function('z')(t)
        a=Symbol('a',positive=True)
        ode_eq=z.diff(t)+a*z-sin(10*t)

        odes = cls(ode_eq,dvars=z,ivar=t,ode_order=1)

        return odes
    
class LinearFirstOrderNonHomo(ODESystem):

    @classmethod
    def from_reference_data(cls):
        t = Symbol('t')
        z = Function('z')(t)
        a = Symbol('a',positive=True)
        ode_eq = z.diff(t)+a*z-cos(t)*Heaviside(t-10)

        odes = cls(ode_eq,dvars=z,ivar=t,ode_order=1)

        return odes

class LinearSecondOrder(ODESystem):

    @classmethod
    def from_reference_data(cls):
        t = Symbol('t')
        z= Function('z')(t)
        h=Symbol('h',positive=True)
        omg0=Symbol('omega_0',positive=True)
        fs=Symbol('f_static',positive=True)
        fd=Symbol('f_dynamic',positive=True)
        omg=Symbol('Omega',positive=True)
        ode_eq=z.diff(t,2)+2*h*z.diff(t)+omg0**2*z+fd*cos(omg*t)+fs

        odes = cls(ode_eq,dvars=z,ivar=t,ode_order=2)

        return odes

class LinearSecondOrderExpSin(ODESystem):

    @classmethod
    def from_reference_data(cls):
        t = Symbol('t')
        z= Function('z')(t)
        h=Symbol('h',positive=True)
        omg0=Symbol('omega_0',positive=True)
        fs=Symbol('f_s',positive=True)
        fd=Symbol('f_d',positive=True)
        omg=Symbol('Omega',positive=True)
        a=Symbol('a',positive=True)
        ode_eq=z.diff(t,2)+2*h*z.diff(t)+omg0**2*z-fd*exp(-a*t)*sin(omg*t)-fs

        odes = cls(ode_eq,dvars=z,ivar=t,ode_order=2)

        return odes
    
class LinearSecondOrderExpCos(ODESystem):

    @classmethod
    def from_reference_data(cls):
        t = Symbol('t')
        z= Function('z')(t)
        h=Symbol('h',positive=True)
        omg0=Symbol('omega_0',positive=True)
        fs=Symbol('f_s',positive=True)
        fd=Symbol('f_d',positive=True)
        omg=Symbol('Omega',positive=True)
        a=Symbol('a',positive=True)
        ode_eq=z.diff(t,2)+2*h*z.diff(t)+omg0**2*z-fd*exp(-a*t)*cos(omg*t)-fs

        odes = cls(ode_eq,dvars=z,ivar=t,ode_order=2)

        return odes
    
class LinearSecondOrderHeavisideComp(ODESystem):

    @classmethod
    def from_reference_data(cls):
        t = Symbol('t')
        z= Function('z')(t)
        h=Symbol('h',positive=True)
        omg0=Symbol('omega_0',positive=True)
        fs=Symbol('f_s',positive=True)
        fd=Symbol('f_d',positive=True)
        omg=Symbol('Omega',positive=True)
        ode_eq=z.diff(t,2)+omg0**2*z-fd*Heaviside(sin(omg*t))-fs#+2*h*z.diff(t)

        odes = cls(ode_eq,dvars=z,ivar=t,ode_order=2)

        return odes
    
class LinearSecondOrderSinComp(ODESystem):

    @classmethod
    def from_reference_data(cls):
        t = Symbol('t')
        z= Function('z')(t)
        h=Symbol('h',positive=True)
        omg0=Symbol('omega_0',positive=True)
        fs=Symbol('f_static',positive=True)
        fd=Symbol('f_dynamic',positive=True)
        omg=Symbol('Omega',positive=True)
        ode_eq=z.diff(t,2)+2*h*z.diff(t)+omg0**2*z+fd*sin(omg*t)+fs

        odes = cls(ode_eq,dvars=z,ivar=t,ode_order=2)

        return odes


    @classmethod
    def from_reference_data(cls):
        t = Symbol('t')
        z= Function('z')(t)
        h=Symbol('h',positive=True)
        omg0=Symbol('omega_0',positive=True)
        fs=Symbol('f_s',positive=True)
        fd=Symbol('f_d',positive=True)
        omg=Symbol('Omega',positive=True)
        ode_eq=z.diff(t,2)+2*h*z.diff(t)+omg0**2*z-fd*exp(-h*t)*sin(omg*t)-fs

        odes = cls(ode_eq,dvars=z,ivar=t,ode_order=2)

        return odes
    
class LinearSecondOrderExpCos(ODESystem):

    @classmethod
    def from_reference_data(cls):
        t = Symbol('t')
        z= Function('z')(t)
        h=Symbol('h',positive=True)
        omg0=Symbol('omega_0',positive=True)
        fs=Symbol('f_s',positive=True)
        fd=Symbol('f_d',positive=True)
        omg=Symbol('Omega',positive=True)
        ode_eq=z.diff(t,2)+2*h*z.diff(t)+omg0**2*z-fd*exp(-h*t)*cos(omg*t)-fs

        odes = cls(ode_eq,dvars=z,ivar=t,ode_order=2)

        return odes
    
class LinearSecondOrderHeavisideComp(ODESystem):

    @classmethod
    def from_reference_data(cls):
        t = Symbol('t')
        z= Function('z')(t)
        h=Symbol('h',positive=True)
        omg0=Symbol('omega_0',positive=True)
        fs=Symbol('f_s',positive=True)
        fd=Symbol('f_d',positive=True)
        omg=Symbol('Omega',positive=True)
        ode_eq=z.diff(t,2)+2*h*z.diff(t)+omg0**2*z-fd*Heaviside(omg*t)-fs

        odes = cls(ode_eq,dvars=z,ivar=t,ode_order=2)

        return odes

class LinearHorizontalSystem(ODESystem):

    
    @classmethod
    def from_reference_data(cls):
        t = Symbol('t')
        z= Function('z')(t)
        h=Symbol('h',positive=True)
        omg0=Symbol('omega_0',positive=True)
        ode_eq=z.diff(t,2)+2*h*z.diff(t)+omg0**2*z

        odes = cls(ode_eq,dvars=z,ivar=t,ode_order=2)

        return odes



#gotowe!!!
class SpringMassSystem(ODESystem):


    @classmethod
    def from_reference_data(cls):
        t = Symbol('t')
        x= Function('x')(t)
        c=Symbol('c',positive=True)
        k=Symbol('k',positive=True)
        m=Symbol('m',positive=True)
        F=Symbol('F',positive=True)
        Omega=Symbol('Omega', positive=True)
        ode_eq=Eq(m*x.diff(t,2)+c*x.diff(t)+k*x,F*sin(Omega*t))

        odes = cls(ode_eq.lhs-ode_eq.rhs,dvars=x,ivar=t,ode_order=2)

        return odes

class SpringMassSmallParameter(ODESystem):


    @classmethod
    def from_reference_data(cls):
        t = Symbol('t')
        x= Function('x')(t)
        eps=Symbol('\epsilon')
        omega=Symbol('\omega',positive=True)
        ode_eq=Eq(x.diff(t,2) + eps*diff(x) + omega**2*x,0)

        odes = cls(ode_eq.lhs-ode_eq.rhs,dvars=x,ivar=t,ode_order=2)

        return odes

class SpringMassDamperMSM(MultiTimeScaleSolution):


    @classmethod
    def from_reference_data(cls):
        t = Symbol('t')
        x= Function('x')(t)
        eps=Symbol('\epsilon')
        omega=Symbol('\omega',positive=True)
        ode_eq=Eq(x.diff(t,2) + eps*diff(x) + omega**2*x,0)

        odes = cls(ode_eq.lhs-ode_eq.rhs,dvars=x, eps=eps)

        return odes

class SpringMassDamperExcitedMSM(MultiTimeScaleSolution):


    @classmethod
    def from_reference_data(cls):
        t = Symbol('t')
        x= Function('x')(t)
        eps=Symbol('\epsilon')
        omega=Symbol('\omega',positive=True)
        omega_0 = Symbol('omega_0',positive = True)
        Omega = Symbol('Omega', positive = True)
        A = Symbol('A', positive = True)
        delta = Symbol('delta', positive = True)
        k=Symbol('k',positive=True)
        m=Symbol('m',positive=True)
        mu=Symbol('mu',positive=True)
        
        MSM_eq=Eq(x.diff(t,2) + eps*diff(x) + omega_0**2*x,eps*mu*sin(Omega * t))
        MSM = cls(MSM_eq.lhs-MSM_eq.rhs,dvars=x, eps=eps)
#         exc_dict = {omega_0:Omega**2+eps*delta}
#         MSM_sol_sub = MSM.solution.subs(exc_dict)
        
        return MSM
    
class SpringMassDamperExcitedSP(ODESystem):


    @classmethod
    def from_reference_data(cls):
        t = Symbol('t')
        x= Function('x')(t)
        eps=Symbol('\epsilon')
        omega=Symbol('\omega',positive=True)
        omega_0 = Symbol('omega_0',positive = True)
        Omega = Symbol('Omega', positive = True)
        A = Symbol('A', positive = True)
        delta = Symbol('delta', positive = True)
        k=Symbol('k',positive=True)
        m=Symbol('m',positive=True)
        mu=Symbol('mu',positive=True)
        
        MSM_eq=Eq(x.diff(t,2) + eps*diff(x) + omega_0**2*x,eps*mu*sin(Omega * t))
        MSM = cls(MSM_eq.lhs-MSM_eq.rhs,dvars=x)
#         exc_dict = {omega_0:Omega**2+eps*delta}
#         MSM_sol_sub = MSM.solution.subs(exc_dict)
        
        return MSM
    
class SpringMassExcitedMSM(ODESystem):


    @classmethod
    def from_reference_data(cls):
        t = Symbol('t')
        x= Function('x')(t)
        eps=Symbol('\epsilon')
        omega=Symbol('\omega',positive=True)
        omega_0 = Symbol('omega_0',positive = True)
        Omega = Symbol('Omega', positive = True)
        A = Symbol('A', positive = True)
        delta = Symbol('delta', positive = True)
        k=Symbol('k',positive=True)
        m=Symbol('m',positive=True)
        
        MSM_eq=Eq(x.diff(t,2) + omega_0**2*x,eps*sin(Omega * t))
        MSM = cls(MSM_eq.lhs-MSM_eq.rhs,dvars=x)
#         exc_dict = {omega_0:Omega**2+eps*delta}
#         MSM_sol_sub = MSM.solution.subs(exc_dict)
        
        return MSM


class HarmonicOscillator(ODESystem):


    @classmethod
    def from_reference_data(cls):
        system=dyn_sys=ForcedSpringMassSystem()
        sim_time=250
        t_span = np.linspace(1,sim_time,sim_time+1)
        ic_list = [0.0,0.0]
        sym_num=18
        sym_list=np.linspace(0,int((sym_num*2)-2),int(sym_num))
        k_list=np.linspace(0.1,1.5,sym_num)
        omg=Symbol("omega")
        param_dict={system.F:1*cos(omg*system.ivar),system.k:omg,system.g:0,system.m:1}
        na_df = dyn_sys.subs(param_dict).eoms.numerical_analysis(parameter=omg,param_span=k_list, t_span = t_span)
        na_df_sol = na_df.compute_solution(t_span = t_span, ic_list = [0.1,0.1]).iloc[:,sym_list]
        plot=na_df_sol.plot()
        plt.show()
        max_plot=na_df_sol.set_axis(k_list,axis=1).max().plot()
        plt.show()

        return plot, max_plot

class ParametricOscillator(ODESystem):


    @classmethod
    def from_reference_data(cls):
        system=dyn_sys=ForcedSpringMassSystem()
        sim_time=250
        t_span = np.linspace(1,sim_time,sim_time+1)
        ic_list = [0.0,0.0]
        sym_num=18
        sym_list=np.linspace(0,int((sym_num*2)-2),int(sym_num))
        k_list=np.linspace(0.1,1.5,sym_num)
        omg=Symbol("omega")
        param_dict_k={system.k:1+0.01*cos(omg*system.ivar),system.F:1*cos(omg*system.ivar),system.g:0,system.m:1}
        na_df_k = dyn_sys.subs(param_dict_k).eoms.numerical_analysis(parameter=omg,param_span=k_list, t_span = t_span)
        na_df_sol_k = na_df_k.compute_solution(t_span = t_span, ic_list = [0.1,0.1]).iloc[:,sym_list]
        plot=na_df_sol_k.plot()
        plt.show()
        max_plot=na_df_sol_k.set_axis(k_list,axis=1).max().plot()
        plt.show()

        return plot, max_plot





from sympy import (Symbol, symbols, Matrix, sin, cos, asin, diff, sqrt, S,
                   diag, Eq, hessian, Function, flatten, Tuple, im, pi, latex,
                   dsolve, solve, fraction, factorial, Subs, Number, oo, Abs,
                   N, solveset)

from sympy.physics.mechanics import dynamicsymbols, ReferenceFrame, Point
from sympy.physics.vector import vpprint, vlatex
from ...dynamics import LagrangesDynamicSystem, HarmonicOscillator, mech_comp

from ..elements import MaterialPoint, Spring, GravitationalForce, Disk, RigidBody2D, Damper, PID, Excitation, Force, base_frame, base_origin
from dynpy import * # enables mechanical models for mathematical modelling


from sympy import * # provides mathematical interface for symbolic calculations


from sympy.physics.mechanics import *

import sympy as sym
import numpy as np
import numpy.fft as fft

import matplotlib.pyplot as plt
import math

from dynpy.utilities.report import SystemDynamicsAnalyzer

import pandas as pd

from dynpy.utilities.adaptable import *


from pint import UnitRegistry
ureg = UnitRegistry()

ix = pd.IndexSlice



mechanics_printing(pretty_print=True)

t,f= symbols('t, f')


from .principles import ComposedSystem, NonlinearComposedSystem, base_frame, base_origin, REPORT_COMPONENTS_LIST


#TODO 151
class FourDOFTrolleySuspension(ComposedSystem):

    scheme_name = 'car.PNG'
    real_name = 'car_real.jpg'
    
    z,phi,z_pw,z_lw=dynamicsymbols('z, \\varphi z_pw z_lw')
    m=Symbol('m', positive=True)
    m_b=Symbol('m_b', positive=True)
    m_k=Symbol('m_k', positive=True)
    m_p=Symbol('m_p', positive=True)
    m_l=Symbol('m_l', positive=True)
    I=Symbol('I', positive=True)
    l_rod=Symbol('l_{rod}', positive=True)
    l_l=Symbol('l_l', positive=True)
    l_k=Symbol('l_k', positive=True)
    l_p=Symbol('l_p', positive=True)
    l_b=Symbol('l_b', positive=True)
    k_l=Symbol('k_l1', positive=True)
    k_lw=Symbol('k_lw', positive=True)
    k_p=Symbol('k_p1', positive=True)
    k_pw=Symbol('k_pw', positive=True)
    c_l=Symbol('c_l', positive=True)
    c_p=Symbol('c_p', positive=True)
    c_lw=Symbol('c_lw', positive=True)
    c_pw=Symbol('c_pw', positive=True)
    F_engine=Symbol('F_{engine}', positive=True)
    k_p=Symbol('k_p', positive=True)
    k_l=Symbol('k_l', positive=True)
    A=Symbol('A', positive=True)
    omega=Symbol('omega', positive=True)
    z_l=Symbol('z_l', positive=True)
    z_p=Symbol('z_p',positive=True)
    z_b=Symbol('z_b',positive=True)
    s=Symbol('s', positive=True)
    ivar=Symbol('t')
    f=Symbol('f', positive=True)
    
    
    def __init__(self,
                 m=None,
                 m_k=None,
                 m_p=None,
                 m_l=None,
                 m_b=None,
                 I=None,
                 l_rod=None,
                 l_l=None,
                 l_p=None,
                 l_b=None,
                 l_k=None,
                 k_l=None,
                 k_lw=None,
                 k_p=None,
                 k_pw=None,
                 c_l=None,
                 c_p=None,
                 c_lw=None,
                 c_pw=None,
                 f=None,
                 F_engine=None,
                 ivar=Symbol('t'),
                 z=None,
                 s=None,
                 phi=None,
                 z_pw=None,
                 z_lw=None,
                 A=None,
                 omega=None,
                 z_l=None,
                 z_p=None,
                 z_b=None,
            
                
            
                 **kwargs):
        
        if z is not None: self.z=z
        if z_lw is not None: self.z_lw=z_lw
        if z_pw is not None: self.z_pw=z_pw
        if z_l is not None: self.z_lw=z_lw
        if z_p is not None: self.z_pw=z_pw
        if z_b is not None: self.z_b=z_b
        if phi is not None: self.phi=phi

        if m is not None: self.m = m  # mass of a rod
        if m_p is not None: self.m_p = m_p
        if m_l is not None: self.m_l = m_l
        if m_b is not None: self.m_b = m_b
        if m_k is not None: self.m_k = m_k
        if l_l is not None: self.l_l = l_l  # offset of left spring
        if l_p is not None: self.l_p = l_p #offset of right spring
        if l_b is not None: self.l_b = l_b  
        if l_k is not None: self.l_k = l_k  
        if l_rod is not None: self.l_rod = l_rod
        if k_l is not None: self.k_l1 = k_l1
        if k_lw is not None: self.k_lw = k_lw
        if k_p is not None: self.k_p1 = k_p1
        if k_pw is not None: self.k_pw = k_pw
        if c_l is not None: self.c_l=c_l
        if c_p is not None: self.c_p=c_p
        if c_lw is not None: self.c_lw=c_lw
        if c_pw is not None: self.c_pw=c_pw
        if A is not None: self.A=A
        if f is not None: self.f=f
        if omega is not None: self.omega=omega
        if I is not None: self.I = I  # moment of inertia of a rod
        if F_engine is not None: self.F_engine = F_engine

        
        self.s=self.A*sin(ivar*self.omega)
        
       
        
        self.right_wheel = MaterialPoint(self.m_p, pos1=self.z_pw, qs=[self.z_pw])
        self.left_wheel = MaterialPoint(self.m_l, pos1=self.z_lw, qs=[self.z_lw])
        self.spring_pw = Spring(self.k_pw, pos1=self.z_pw, pos2=self.s, qs=[self.z_pw])
        self.spring_lw = Spring(self.k_lw, pos1=self.z_lw, pos2=self.s, qs=[self.z_lw])
        self.damper_pw = Damper(self.c_pw, pos1=self.z_pw, pos2=self.s, qs=[self.z_pw])
        self.damper_lw = Damper(self.c_lw, pos1=self.z_lw, pos2=self.s, qs=[self.z_lw])
        
   
        
        self.body = RigidBody2D(self.m, self.I, pos_lin=self.z, pos_rot=self.phi, qs=[self.z, self.phi])  # rod
        self.battery= MaterialPoint(self.m_b, pos1=self.z-self.phi*self.l_b, qs=[self.z, self.phi])
#         self.pasazer= MaterialPoint(self.m_k, pos1=self.z+self.phi*self.l_k, qs=[self.z, self.phi])
        self.spring_1 = Spring(self.k_l, pos1=self.z+self.phi*self.l_l , pos2 = self.z_lw , qs=[self.z, self.phi, self.z_lw])  # left spring
        self.spring_2 = Spring(self.k_p, pos1=self.z-self.phi*self.l_p , pos2 = self.z_pw , qs=[self.z, self.phi, self.z_pw])
        self.damper_1= Damper(self.c_l, pos1=self.z+self.phi*self.l_l , pos2 = self.z_lw , qs=[self.z, self.phi, self.z_lw])
        self.damper_2 = Damper(self.c_p, pos1=self.z-self.phi*self.l_p , pos2 = self.z_pw , qs=[self.z, self.phi, self.z_pw])
        #self.force_1 = Force(self.A*sin(ivar*self.omega), pos1=self.z_lw , qs=[self.z_lw])
        #self.force_2 = Force(self.A*sin(ivar*self.omega), pos1=self.z_pw, qs=[self.z_pw])
        
        #self.force = Force(-self.F_engine, pos1=self.z - self.l_p * self.phi, qs=[self.z, self.phi])
        system = self.body +self.battery+self.spring_1 + self.spring_2 +self.damper_1+self.damper_2 + self.right_wheel + self.left_wheel + self.spring_pw+ self.spring_lw+self.damper_lw+self.damper_pw

        super().__init__(**{'system':system,**kwargs})
        


    def symbols_description(self):
        self.sym_desc_dict = {
            self.m: r'Masa resorowana',
            self.m_p: r'Masa przedniej osi',
            self.m_l: r'Masa tylnej osi',
            self.m_b: r'Masa baterii trakcyjnej',
            self.m_b: r'Masa pasazera',
            self.I: r'Moment bezwładności',
            self.l_rod: r'Długość osi',
            self.l_p: r'Odległość środka ciężkości masy nadwozia autobusu do punktu styku przednich kół w kierunku poziomym',
            self.l_l: r'Odległość środka ciężkości masy nadwozia autobusu do punktu styku tylnych kół w kierunku poziomym',
            self.l_b: r'Odległość środka ciężkości masy nadwozia autobusu do środka ciężkości baterii',
            self.k_p: r'Współczynnik sztywności dla sprężyny przedniej nadwozia',
            self.k_l: r'Współczynnik sztywności dla sprężyny tylnej nadwozia',
            self.k_pw: r'Współczynnik sztywności dla przedniej opony',
            self.k_lw:  r'Współczynnik sztywności dla tylnej opony',
            self.c_l:  r'Współczynnik tłumienia dla amortzatora przedniego n',
            self.c_l: r'Współczynnik tłumienia dla amortzatora tylnego',
            self.c_lw:  r'Współczynnik tłumienia dla tylnej opony',
            self.c_pw:  r'Współczynnik tłumienia dla przedniej opony',
            self.phi:  r'Kąt obrotu masy resorowanej',
            self.z: r'Przemieszczenie pionowe środka cieżkości masy resorowanej',
            self.z_pw:  r'Przemieszczenie pionowe przedniego koła',
            self.z_lw: r'Przemieszczenie pionowe tylnego koła',
            self.z_b: r'Przemieszczenie pionowe baterii',
            self.A: r'Amplituda siły wymuszającej',
            self.omega: r'Częstość siły wymuszającej',
            self.F_engine: r'Siła',
            self.ivar: r'Czas',
            self.f: r'Częstotliwość wymuszenia',
            
            
        }
        return self.sym_desc_dict
    
sys4=FourDOFTrolleySuspension()
y=[sys4.q,sys4.z, sys4.z.diff(t,t)]
ic_list = [0.1,0.1,0,0,0.1,0.1,0,0]


units_dict = {sys4.m:ureg.kilogram,
              sys4.m_p:ureg.kilogram,
              sys4.m_l:ureg.kilogram,
              sys4.m_b:ureg.kilogram,
              sys4.m_k:ureg.kilogram,
              sys4.A:ureg.meter,
              sys4.k_l:(ureg.newton / ureg.meter),
              sys4.k_lw:(ureg.newton / ureg.meter),
              sys4.k_p:(ureg.newton / ureg.meter),
              sys4.k_pw:(ureg.newton / ureg.meter),
              sys4.c_l:(ureg.newton*ureg.second / ureg.meter),
              sys4.c_lw:(ureg.newton*ureg.second / ureg.meter),
              sys4.c_p:(ureg.newton*ureg.second / ureg.meter),
              sys4.c_pw:(ureg.newton*ureg.second / ureg.meter),
              sys4.omega:ureg.radian,
              sys4.l_l:ureg.meter,
              sys4.l_p:ureg.meter,
              sys4.l_b:ureg.meter,
              sys4.l_rod:ureg.meter,
              sys4.I:(ureg.kilogram*ureg.meter*ureg.meter),
              sys4.phi:ureg.radian,
              sys4.z:ureg.meter,
              sys4.z.diff(t,t):ureg.meter/ureg.second**2,
              sys4.z_lw:ureg.meter,
              sys4.z_pw:ureg.meter,
              sys4.z_b:ureg.meter,
              sys4.F_engine:ureg.newton,
              t:ureg.second,
              f:ureg.hertz,
              
             }

unit=units_dict
#TODO        
class DDOFTrolleySuspension(ComposedSystem):

   
    
    z,phi,z_pw,z_lw=dynamicsymbols('z, \\varphi z_pw z_lw')
    m=Symbol('m', positive=True)
    m_b=Symbol('m_b', positive=True)
    m_p=Symbol('m_p', positive=True)
    m_l=Symbol('m_l', positive=True)
    I=Symbol('I', positive=True)
    l_rod=Symbol('l_{rod}', positive=True)
    l_l=Symbol('l_l', positive=True)
    l_p=Symbol('l_p', positive=True)
    l_b=Symbol('l_b', positive=True)
    k_l=Symbol('k_l1', positive=True)
    k_lw=Symbol('k_lw', positive=True)
    k_p=Symbol('k_p1', positive=True)
    k_pw=Symbol('k_pw', positive=True)
    c_l=Symbol('c_l', positive=True)
    c_p=Symbol('c_p', positive=True)
    c_lw=Symbol('c_lw', positive=True)
    c_pw=Symbol('c_pw', positive=True)
    F_engine=Symbol('F_{engine}', positive=True)
    k_p=Symbol('k_p', positive=True)
    k_l=Symbol('k_l', positive=True)
    A=Symbol('A', positive=True)
    omega=Symbol('omega', positive=True)
    z_l=Symbol('z_l', positive=True)
    z_p=Symbol('z_p',positive=True)
    z_b=Symbol('z_b',positive=True)
    s=Symbol('s', positive=True)
    ivar=Symbol('t')
    k_mr=Symbol('k_mr',positive=True)
    m_nr=Symbol('m_nr')
    
    def __init__(self,
                  m=None,
                 m_p=None,
                 m_l=None,
                 m_b=None,
                 I=None,
                 l_rod=None,
                 l_l=None,
                 l_p=None,
                 l_b=None,
                 k_l=None,
                 k_lw=None,
                 k_p=None,
                 k_pw=None,
                 c_l=None,
                 c_p=None,
                 c_lw=None,
                 c_pw=None,
                 F_engine=None,
                 ivar=Symbol('t'),
                 z=None,
                 s=None,
                 phi=None,
                 z_pw=None,
                 z_lw=None,
                 A=None,
                 omega=None,
                 z_l=None,
                 z_p=None,
                 z_b=None,
                 k_mr=None,
                 m_nr=None,
                 **kwargs):
        
        if z is not None: self.z=z
        if phi is not None: self.phi=phi
        

        if m is not None: self.m = m  # mass of a rod
        if m_p is not None: self.m_w = m_w
        if m_l is not None: self.m_w = m_w
        if m_b is not None: self.m_b = m_b
        if l_l is not None: self.l_l = l_l  # offset of left spring
        if l_p is not None: self.l_p = l_p #offset of right spring
        if l_b is not None: self.l_p = l_b  
        if l_rod is not None: self.l_rod = l_rod
        if k_l is not None: self.k_l1 = k_l1
        if k_lw is not None: self.k_lw = k_lw
        if k_p is not None: self.k_p1 = k_p1
        if k_pw is not None: self.k_pw = k_pw
        if c_l is not None: self.c_l=c_l
        if c_p is not None: self.c_p=c_p
        if c_lw is not None: self.c_lw=c_lw
        if c_pw is not None: self.c_pw=c_pw
        if A is not None: self.A=A
        if omega is not None: self.omega=omega
        if I is not None: self.I = I  # moment of inertia of a rod
        if F_engine is not None: self.F_engine = F_engine     
        if k_mr is not None: self.k_mr=k_mr
        if m_nr is not None: self.m_nr=m_nr
            
            
        self.s=self.A*sin(ivar*self.omega)
        
        self.z_l=self.z+self.phi*self.l_l
        self.z_p=self.z-self.phi*self.l_p
        self.z_lw=(self.k_l*self.z_l)/(self.k_lw+self.k_l)
        self.z_pw=(self.k_p*self.z_p)/(self.k_pw+self.k_p)
        
        
        self.right_wheel = MaterialPoint(self.m_p, pos1=self.z_pw, qs=[self.z, self.phi])
        self.left_wheel = MaterialPoint(self.m_l, pos1=self.z_lw, qs=[self.z, self.phi])
        self.spring_pw = Spring(self.k_pw, pos1=self.z_pw, pos2=self.s, qs=[self.z, self.phi])
        self.spring_lw = Spring(self.k_lw, pos1=self.z_lw, pos2=self.s, qs=[self.z, self.phi])
        self.damper_pw = Damper(self.c_pw, pos1=self.z_pw, pos2=self.s, qs=[self.z, self.phi])
        self.damper_lw = Damper(self.c_lw, pos1=self.z_lw, pos2=self.s, qs=[self.z, self.phi])
        #self.spring=Spring(k,pos1,po2,qs)
        
        self.body = RigidBody2D(self.m, self.I, pos_lin=self.z, pos_rot=self.phi, qs=[self.z, self.phi])  # rod
        self.battery= MaterialPoint(self.m_b, pos1=self.z+self.phi*self.l_b, qs=[self.z, self.phi])
        self.spring_1 = Spring(self.k_l, pos1=self.z_l , pos2 = self.z_lw , qs=[self.z, self.phi])  # left spring
        self.spring_2 = Spring(self.k_p, pos1=self.z_p, pos2=self.z_pw, qs=[self.z, self.phi])
        self.damper_1=Damper(self.c_l, pos1=self.z_p , pos2 = self.z_lw , qs=[self.z, self.phi])
        self.damper_2 = Damper(self.c_p, pos1=self.z_l , pos2 = self.z_pw , qs=[self.z, self.phi])

        #self.force = Force(-self.F_engine, pos1=self.z - self.l_p * self.phi, qs=[self.z, self.phi])
        system = self.body +self.battery+ self.spring_1 + self.spring_2 + self.damper_1+ self.damper_2 + self.right_wheel + self.left_wheel + self.spring_pw + self.spring_lw+self.damper_lw+self.damper_pw
        
#         display(type(system))
        
#         system_new = system.subs({self.z_lw:self.z_l2,self.z_pw:self.z_p2}) 
        
#         display(type(system_new))
#         display(type(self.body))
        
        #super().__init__(system_new,**kwargs)

        super().__init__(**{'system':system,**kwargs})
        


        
    def symbols_description(self):
        self.sym_desc_dict = {
            self.m: r'Masa resorowana',
            self.m_p: r'Masa przedniej osi',
            self.m_l: r'Masa tylnej osi',
            self.m_b: r'Masa baterii trakcyjnej',
            self.I: r'Moment bezwładności',
            self.l_rod: r'Długość osi',
            self.l_p: r'Odległość od przedniej osi do środka ciężkości autobusu',
            self.l_l: r'Odległość od tylnej osi do środka ciężkości autobusu',
            self.l_b: r'Odległość od środka cieżkości baterii do środka ciężkości autobusu',
            self.k_p: r'Współczynnik sztywności dla sprężyny przedniej nadwozia',
            self.k_l: r'Współczynnik sztywności dla sprężyny tylnej nadwozia',
            self.k_pw: r'Współczynnik sztywności dla przedniej opony',
            self.k_lw:  r'Współczynnik sztywności dla tylnej opony',
            self.c_l:  r'Współczynnik tłumienia dla amortzatora przedniego n',
            self.c_l: r'Współczynnik tłumienia dla amortzatora tylnego',
            self.c_lw:  r'Współczynnik tłumienia dla tylnej opony',
            self.c_pw:  r'Współczynnik tłumienia dla przedniej opony',
            self.phi:  r'Kąt obrotu masy resorowanej',
            self.z: r'Przemieszczenie pionowe środka cieżkości masy resorowanej',
            self.z_pw:  r'Przemieszczenie pionowe przedniego koła',
            self.z_lw: r'Przemieszczenie pionowe tylnego koła',
            self.z_b: r'Przemieszczenie pionowe baterii',
            self.A: r'Amplituda siły wymuszającej',
            self.omega: r'Częstość siły wymuszającej',
            self.ivar: r'Czas',
            self.k_mr: r'Zastępcza sztywność mas resorowanych',
            self.m_nr: r'Zastępcza masa nierosowana',
            
        }
        return self.sym_desc_dict
    
sys2=DDOFTrolleySuspension()



units_dict1 = {sys2.m:ureg.kilogram,
              sys2.m_p:ureg.kilogram,
              sys2.m_l:ureg.kilogram,
              sys2.m_b:ureg.kilogram,
              sys2.A:ureg.meter,
              sys2.k_l:(ureg.newton / ureg.meter),
              sys2.k_lw:(ureg.newton / ureg.meter),
              sys2.k_p:(ureg.newton / ureg.meter),
              sys2.k_pw:(ureg.newton / ureg.meter),
              sys2.c_l:(ureg.newton*ureg.second / ureg.meter),
              sys2.c_lw:(ureg.newton*ureg.second / ureg.meter),
              sys2.c_p:(ureg.newton*ureg.second / ureg.meter),
              sys2.c_pw:(ureg.newton*ureg.second / ureg.meter),
              sys2.omega:ureg.radian,
              sys2.l_l:ureg.meter,
              sys2.l_p:ureg.meter,
              sys2.l_b:ureg.meter,
              sys2.l_rod:ureg.meter,
              sys2.I:(ureg.kilogram*ureg.meter*ureg.meter),
              sys2.phi:ureg.radian,
              sys2.z:ureg.meter,
              sys2.z.diff(t,t):ureg.meter/ureg.second**2,
              sys2.z_lw:ureg.meter,
              sys2.z_pw:ureg.meter,
              sys2.z_b:ureg.meter,
              sys2.F_engine:ureg.newton,
              t:ureg.second,
              f:ureg.hertz,
              sys2.k_mr:(ureg.newton / ureg.meter),
              sys2.m_nr:ureg.kilogram,
             }

unit1=units_dict1
        #TODO 153
class DDOFTrolleySuspension2(ComposedSystem):

   
    
    z,phi,z_pw,z_lw=dynamicsymbols('z, \\varphi z_pw z_lw')
    m=Symbol('m', positive=True)
    m_b=Symbol('m_b', positive=True)
    m_p=Symbol('m_p', positive=True)
    m_l=Symbol('m_l', positive=True)
    I=Symbol('I', positive=True)
    l_rod=Symbol('l_{rod}', positive=True)
    l_l=Symbol('l_l', positive=True)
    l_p=Symbol('l_p', positive=True)
    l_b=Symbol('l_b', positive=True)
    k_l=Symbol('k_l1', positive=True)
    k_lw=Symbol('k_lw', positive=True)
    k_p=Symbol('k_p1', positive=True)
    k_pw=Symbol('k_pw', positive=True)
    c_l=Symbol('c_l', positive=True)
    c_p=Symbol('c_p', positive=True)
    c_lw=Symbol('c_lw', positive=True)
    c_pw=Symbol('c_pw', positive=True)
    F_engine=Symbol('F_{engine}', positive=True)
    k_p=Symbol('k_p', positive=True)
    k_l=Symbol('k_l', positive=True)
    A=Symbol('A', positive=True)
    omega=Symbol('omega', positive=True)
    z_l=Symbol('z_l', positive=True)
    z_p=Symbol('z_p',positive=True)
    z_b=Symbol('z_b',positive=True)
    s=Symbol('s', positive=True)
    k_l_zas=Symbol('k_l_zas', positive=True)
    k_p_zas=Symbol('k_p_zas', positive=True)
    c_l_zas=Symbol('c_l_zas', positive=True)
    c_p_zas=Symbol('c_lp_zas', positive=True)
    ivar=Symbol('t')
    
    def __init__(self,
                 m=None,
                 m_p=None,
                 m_l=None,
                 m_b=None,
                 I=None,
                 l_rod=None,
                 l_l=None,
                 l_p=None,
                 l_b=None,
                 k_l=None,
                 k_lw=None,
                 k_p=None,
                 k_pw=None,
                 c_l=None,
                 c_p=None,
                 c_lw=None,
                 c_pw=None,
                 F_engine=None,
                 ivar=Symbol('t'),
                 z=None,
                 phi=None,
                 z_pw=None,
                 z_lw=None,
                 A=None,
                 omega=None,
                 z_l=None,
                 z_p=None,
                 z_b=None,
                 s=None,
                 k_l_zas=None,
                 k_p_zas=None,
                 c_l_zas=None,
                 c_p_zas=None,
                 
                 **kwargs):
        if z is not None: self.z=z
        if z_lw is not None: self.z_lw=z_lw
        if z_pw is not None: self.z_pw=z_pw
        if z_l is not None: self.z_lw=z_lw
        if z_p is not None: self.z_pw=z_pw
        if z_b is not None: self.z_b=z_b
        if phi is not None: self.phi=phi

        if m is not None: self.m = m  # mass of a rod
        if m_p is not None: self.m_w = m_w
        if m_l is not None: self.m_w = m_w
        if m_b is not None: self.m_b = m_b
        if l_l is not None: self.l_l = l_l  # offset of left spring
        if l_p is not None: self.l_p = l_p #offset of right spring
        if l_b is not None: self.l_p = l_b  
        if l_rod is not None: self.l_rod = l_rod
        if k_l is not None: self.k_l1 = k_l1
        if k_lw is not None: self.k_lw = k_lw
        if k_p is not None: self.k_p1 = k_p1
        if k_pw is not None: self.k_pw = k_pw
        if c_l is not None: self.c_l=c_l
        if c_p is not None: self.c_p=c_p
        if c_lw is not None: self.c_lw=c_lw
        if c_pw is not None: self.c_pw=c_pw
        if A is not None: self.A=A
        if omega is not None: self.omega=omega
        if I is not None: self.I = I  # moment of inertia of a rod
        if F_engine is not None: self.F_engine = F_engine
        
                 
        self.s=self.A*sin(ivar*self.omega)
        
        self.k_l_zas=(self.k_l*self.k_lw)/(self.k_l+self.k_lw)
        self.k_p_zas=(self.k_p*self.k_pw)/(self.k_p+self.k_pw)
        self.c_l_zas=(self.c_l*self.c_lw)/(self.c_l+self.c_lw)
        self.c_p_zas=(self.c_p*self.c_pw)/(self.c_p+self.c_pw)
        
        self.z_l=self.z+self.phi*self.l_l
        self.z_p=self.z-self.phi*self.l_p
        self.z_lw=(self.k_l*self.z_l)/(self.k_lw+self.k_l)
        self.z_pw=(self.k_p*self.z_p)/(self.k_pw+self.k_p)
        
        #self.right_wheel = MaterialPoint(self.m_w, pos1=self.z_pw, qs=[self.z, self.phi])
        #self.left_wheel = MaterialPoint(self.m_w, pos1=self.z_lw, qs=[self.z, self.phi])
        #self.spring_pw = Spring(self.k_pw, pos1=self.z_pw, pos2=self.s, qs=[self.z, self.phi])
        #self.spring_lw = Spring(self.k_lw, pos1=self.z_lw, pos2=self.s, qs=[self.z, self.phi])
        #self.spring=Spring(k,pos1,po2,qs)
        
        self.body = RigidBody2D(self.m, self.I, pos_lin=self.z, pos_rot=self.phi, qs=[self.z, self.phi])  # rod
        self.battery= MaterialPoint(self.m_b, pos1=self.z+self.phi*self.l_b, qs=[self.z, self.phi])
        self.spring_1 = Spring(self.k_l_zas, pos1=self.z_l , pos2 = self.s , qs=[self.z, self.phi])  # left spring
        self.spring_2 = Spring(self.k_p_zas, pos1=self.z_p, pos2=self.s, qs=[self.z, self.phi])
        self.damper_1=Damper(self.c_l_zas, pos1=self.z_p , pos2 = self.s, qs=[self.z, self.phi])
        self.damper_2 = Damper(self.c_p_zas, pos1=self.z_l , pos2 = self.s , qs=[self.z, self.phi])

        #self.force = Force(-self.F_engine, pos1=self.z - self.l_p * self.phi, qs=[self.z, self.phi])
        system = self.body +self.battery+ self.spring_1 + self.spring_2 + self.damper_1+ self.damper_2 
        
#         display(type(system))
        
#         system_new = system.subs({self.z_lw:self.z_l2,self.z_pw:self.z_p2}) 
        
#         display(type(system_new))
#         display(type(self.body))
        
        #super().__init__(system_new,**kwargs)

        super().__init__(**{'system':system,**kwargs})        

    def symbols_description(self):
        self.sym_desc_dict = {
            self.m: r'Masa resorowana',
            self.m_p: r'Masa przedniej osi',
            self.m_l: r'Masa tylnej osi',
            self.m_b: r'Masa baterii trakcyjnej',
            self.I: r'Moment bezwładności',
            self.l_rod: r'Długość osi',
            self.l_p: r'Odległość od przedniej osi do środka ciężkości autobusu',
            self.l_l: r'Odległość od tylnej osi do środka ciężkości autobusu',
            self.l_b: r'Odległość od środka cieżkości baterii do środka ciężkości autobusu',
            self.k_p: r'Współczynnik sztywności dla sprężyny przedniej nadwozia',
            self.k_l: r'Współczynnik sztywności dla sprężyny tylnej nadwozia',
            self.k_pw: r'Współczynnik sztywności dla przedniej opony',
            self.k_lw:  r'Współczynnik sztywności dla tylnej opony',
            self.c_l:  r'Współczynnik tłumienia dla amortzatora przedniego n',
            self.c_l: r'Współczynnik tłumienia dla amortzatora tylnego',
            self.c_lw:  r'Współczynnik tłumienia dla tylnej opony',
            self.c_pw:  r'Współczynnik tłumienia dla przedniej opony',
            self.phi:  r'Kąt obrotu masy resorowanej',
            self.z: r'Przemieszczenie pionowe środka cieżkości masy resorowanej',
            self.z_pw:  r'Przemieszczenie pionowe przedniego koła',
            self.z_lw: r'Przemieszczenie pionowe tylnego koła',
            self.z_b: r'Przemieszczenie pionowe baterii',
            self.A: r'Amplituda siły wymuszającej',
            self.omega: r'Częstość siły wymuszającej',
            self.ivar: r'Czas',
            self.k_l_zas: r'Zastępcza wartość współczynnika sztywności zawieszenia tylnej osi',
            self.k_p_zas: r'Zastępcza wartość współczynnika sztywności zawieszenia przedniej osi',
            self.c_l_zas: r'Zastępcza wartość współczynnika tłumienia zawieszenia tylnej osi',
            self.c_p_zas: r'Zastępcza wartość współczynnika tłumienia zawieszenia przedniej osi',
        }
        return self.sym_desc_dict
                 
sys22=DDOFTrolleySuspension2()      
                 
units_dict2 = {sys22.m:ureg.kilogram,
              sys22.m_p:ureg.kilogram,
              sys22.m_l:ureg.kilogram,
              sys22.m_b:ureg.kilogram,
              sys22.A:ureg.meter,
              sys22.k_l:(ureg.newton / ureg.meter),
              sys22.k_lw:(ureg.newton / ureg.meter),
              sys22.k_p:(ureg.newton / ureg.meter),
              sys22.k_pw:(ureg.newton / ureg.meter),
              sys22.c_l:(ureg.newton*ureg.second / ureg.meter),
              sys22.c_lw:(ureg.newton*ureg.second / ureg.meter),
              sys22.c_p:(ureg.newton*ureg.second / ureg.meter),
              sys22.c_pw:(ureg.newton*ureg.second / ureg.meter),
              sys22.omega:ureg.radian,
              sys22.l_l:ureg.meter,
              sys22.l_p:ureg.meter,
              sys22.l_b:ureg.meter,
              sys22.l_rod:ureg.meter,
              sys22.I:(ureg.kilogram*ureg.meter*ureg.meter),
              sys22.phi:ureg.radian,
              sys22.z:ureg.meter,
              sys22.z.diff(t,t):ureg.meter/ureg.second**2,
              sys22.z_lw:ureg.meter,
              sys22.z_pw:ureg.meter,
              sys22.z_b:ureg.meter,
              t:ureg.second,
              f:ureg.hertz,
              sys22.F_engine:ureg.newton,
              sys22.k_p_zas:(ureg.newton / ureg.meter),
              sys22.k_l_zas:(ureg.newton / ureg.meter),
              sys22.c_l_zas:(ureg.newton*ureg.second / ureg.meter),
              sys22.c_p_zas:(ureg.newton*ureg.second / ureg.meter),
             }

unit2=units_dict2
   

